# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.linalg._solve_toeplitz, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import numpy.linalg as _mod_numpy_linalg

LinAlgError = _mod_numpy_linalg.LinAlgError
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def asarray(a, dtype, order) -> typing.Any:
    "Convert the input to an array.\n\n    Parameters\n    ----------\n    a : array_like\n        Input data, in any form that can be converted to an array.  This\n        includes lists, lists of tuples, tuples, tuples of tuples, tuples\n        of lists and ndarrays.\n    dtype : data-type, optional\n        By default, the data-type is inferred from the input data.\n    order : {'C', 'F', 'A', 'K'}, optional\n        Memory layout.  'A' and 'K' depend on the order of input array a.\n        'C' row-major (C-style), \n        'F' column-major (Fortran-style) memory representation.\n        'A' (any) means 'F' if `a` is Fortran contiguous, 'C' otherwise\n        'K' (keep) preserve input order\n        Defaults to 'C'.\n    like : array_like\n        Reference object to allow the creation of arrays which are not\n        NumPy arrays. If an array-like passed in as ``like`` supports\n        the ``__array_function__`` protocol, the result will be defined\n        by it. In this case, it ensures the creation of an array object\n        compatible with that passed in via this argument.\n\n        .. note::\n            The ``like`` keyword is an experimental feature pending on\n            acceptance of :ref:`NEP 35 <NEP35>`.\n\n        .. versionadded:: 1.20.0\n\n    Returns\n    -------\n    out : ndarray\n        Array interpretation of `a`.  No copy is performed if the input\n        is already an ndarray with matching dtype and order.  If `a` is a\n        subclass of ndarray, a base class ndarray is returned.\n\n    See Also\n    --------\n    asanyarray : Similar function which passes through subclasses.\n    ascontiguousarray : Convert input to a contiguous array.\n    asfarray : Convert input to a floating point ndarray.\n    asfortranarray : Convert input to an ndarray with column-major\n                     memory order.\n    asarray_chkfinite : Similar function which checks input for NaNs and Infs.\n    fromiter : Create an array from an iterator.\n    fromfunction : Construct an array by executing a function on grid\n                   positions.\n\n    Examples\n    --------\n    Convert a list into an array:\n\n    >>> a = [1, 2]\n    >>> np.asarray(a)\n    array([1, 2])\n\n    Existing arrays are not copied:\n\n    >>> a = np.array([1, 2])\n    >>> np.asarray(a) is a\n    True\n\n    If `dtype` is set, array is copied only if dtype does not match:\n\n    >>> a = np.array([1, 2], dtype=np.float32)\n    >>> np.asarray(a, dtype=np.float32) is a\n    True\n    >>> np.asarray(a, dtype=np.float64) is a\n    False\n\n    Contrary to `asanyarray`, ndarray subclasses are not passed through:\n\n    >>> issubclass(np.recarray, np.ndarray)\n    True\n    >>> a = np.array([(1.0, 2), (3.0, 4)], dtype='f4,i4').view(np.recarray)\n    >>> np.asarray(a) is a\n    False\n    >>> np.asanyarray(a) is a\n    True\n\n    "
    ...

complex128 = _mod_numpy.complex128
float64 = _mod_numpy.float64
def levinson(a, b) -> typing.Any:
    'Solve a linear Toeplitz system using Levinson recursion.\n\n    Parameters\n    ----------\n    a : array, dtype=double or complex128, shape=(2n-1,)\n        The first column of the matrix in reverse order (without the diagonal)\n        followed by the first (see below)\n    b : array, dtype=double  or complex128, shape=(n,)\n        The right hand side vector. Both a and b must have the same type\n        (double or complex128).\n\n    Notes\n    -----\n    For example, the 5x5 toeplitz matrix below should be represented as\n    the linear array ``a`` on the right ::\n\n        [ a0    a1   a2  a3  a4 ]\n        [ a-1   a0   a1  a2  a3 ]\n        [ a-2  a-1   a0  a1  a2 ] -> [a-4  a-3  a-2  a-1  a0  a1  a2  a3  a4]\n        [ a-3  a-2  a-1  a0  a1 ]\n        [ a-4  a-3  a-2  a-1 a0 ]\n\n    Returns\n    -------\n    x : arrray, shape=(n,)\n        The solution vector\n    reflection_coeff : array, shape=(n+1,)\n        Toeplitz reflection coefficients. When a is symmetric Toeplitz and\n        ``b`` is ``a[n:]``, as in the solution of autoregressive systems,\n        then ``reflection_coeff`` also correspond to the partial\n        autocorrelation function.\n    '
    ...

def zeros(shape, dtype=..., order=..., *, like=...) -> typing.Any:
    "zeros(shape, dtype=float, order='C', *, like=None)\n\n    Return a new array of given shape and type, filled with zeros.\n\n    Parameters\n    ----------\n    shape : int or tuple of ints\n        Shape of the new array, e.g., ``(2, 3)`` or ``2``.\n    dtype : data-type, optional\n        The desired data-type for the array, e.g., `numpy.int8`.  Default is\n        `numpy.float64`.\n    order : {'C', 'F'}, optional, default: 'C'\n        Whether to store multi-dimensional data in row-major\n        (C-style) or column-major (Fortran-style) order in\n        memory.\n    like : array_like\n        Reference object to allow the creation of arrays which are not\n        NumPy arrays. If an array-like passed in as ``like`` supports\n        the ``__array_function__`` protocol, the result will be defined\n        by it. In this case, it ensures the creation of an array object\n        compatible with that passed in via this argument.\n\n        .. note::\n            The ``like`` keyword is an experimental feature pending on\n            acceptance of :ref:`NEP 35 <NEP35>`.\n\n        .. versionadded:: 1.20.0\n\n    Returns\n    -------\n    out : ndarray\n        Array of zeros with the given shape, dtype, and order.\n\n    See Also\n    --------\n    zeros_like : Return an array of zeros with shape and type of input.\n    empty : Return a new uninitialized array.\n    ones : Return a new array setting values to one.\n    full : Return a new array of given shape filled with value.\n\n    Examples\n    --------\n    >>> np.zeros(5)\n    array([ 0.,  0.,  0.,  0.,  0.])\n\n    >>> np.zeros((5,), dtype=int)\n    array([0, 0, 0, 0, 0])\n\n    >>> np.zeros((2, 1))\n    array([[ 0.],\n           [ 0.]])\n\n    >>> s = (2,2)\n    >>> np.zeros(s)\n    array([[ 0.,  0.],\n           [ 0.,  0.]])\n\n    >>> np.zeros((2,), dtype=[('x', 'i4'), ('y', 'i4')]) # custom dtype\n    array([(0, 0), (0, 0)],\n          dtype=[('x', '<i4'), ('y', '<i4')])"
    ...

def __getattr__(name) -> typing.Any:
    ...




class Enum(int):
    def __new__(cls, name, value): ...
    def __repr__(self): ...

GL_ACTIVE_ATTRIBUTES = ...
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = ...
GL_ACTIVE_TEXTURE = ...
GL_ACTIVE_UNIFORMS = ...
GL_ACTIVE_UNIFORM_MAX_LENGTH = ...
GL_ALIASED_LINE_WIDTH_RANGE = ...
GL_ALIASED_POINT_SIZE_RANGE = ...
GL_ALPHA = ...
GL_ALPHA_BITS = ...
GL_ALWAYS = ...
GL_ARRAY_BUFFER = ...
GL_ARRAY_BUFFER_BINDING = ...
GL_ATTACHED_SHADERS = ...
GL_BACK = ...
GL_BLEND = ...
GL_BLEND_COLOR = ...
GL_BLEND_DST_ALPHA = ...
GL_BLEND_DST_RGB = ...
GL_BLEND_EQUATION = ...
GL_BLEND_EQUATION_ALPHA = ...
GL_BLEND_EQUATION_RGB = ...
GL_BLEND_SRC_ALPHA = ...
GL_BLEND_SRC_RGB = ...
GL_BLUE_BITS = ...
GL_BOOL = ...
GL_BOOL_VEC2 = ...
GL_BOOL_VEC3 = ...
GL_BOOL_VEC4 = ...
GL_BUFFER_SIZE = ...
GL_BUFFER_USAGE = ...
GL_BYTE = ...
GL_CCW = ...
GL_CLAMP_TO_EDGE = ...
GL_COLOR_ATTACHMENT0 = ...
GL_COLOR_BUFFER_BIT = ...
GL_COLOR_CLEAR_VALUE = ...
GL_COLOR_WRITEMASK = ...
GL_COMPILE_STATUS = ...
GL_COMPRESSED_TEXTURE_FORMATS = ...
GL_CONSTANT_ALPHA = ...
GL_CONSTANT_COLOR = ...
GL_CULL_FACE = ...
GL_CULL_FACE_MODE = ...
GL_CURRENT_PROGRAM = ...
GL_CURRENT_VERTEX_ATTRIB = ...
GL_CW = ...
GL_DECR = ...
GL_DECR_WRAP = ...
GL_DELETE_STATUS = ...
GL_DEPTH_ATTACHMENT = ...
GL_DEPTH_BITS = ...
GL_DEPTH_BUFFER_BIT = ...
GL_DEPTH_CLEAR_VALUE = ...
GL_DEPTH_COMPONENT = ...
GL_DEPTH_COMPONENT16 = ...
GL_DEPTH_FUNC = ...
GL_DEPTH_RANGE = ...
GL_DEPTH_TEST = ...
GL_DEPTH_WRITEMASK = ...
GL_DITHER = ...
GL_DONT_CARE = ...
GL_DST_ALPHA = ...
GL_DST_COLOR = ...
GL_DYNAMIC_DRAW = ...
GL_ELEMENT_ARRAY_BUFFER = ...
GL_ELEMENT_ARRAY_BUFFER_BINDING = ...
GL_EQUAL = ...
GL_ES_VERSION_2_0 = ...
GL_EXTENSIONS = ...
GL_FALSE = ...
GL_FASTEST = ...
GL_FIXED = ...
GL_FLOAT = ...
GL_FLOAT_MAT2 = ...
GL_FLOAT_MAT3 = ...
GL_FLOAT_MAT4 = ...
GL_FLOAT_VEC2 = ...
GL_FLOAT_VEC3 = ...
GL_FLOAT_VEC4 = ...
GL_FRAGMENT_SHADER = ...
GL_FRAMEBUFFER = ...
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = ...
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = ...
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = ...
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = ...
GL_FRAMEBUFFER_BINDING = ...
GL_FRAMEBUFFER_COMPLETE = ...
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = ...
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS = ...
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = ...
GL_FRAMEBUFFER_UNSUPPORTED = ...
GL_FRONT = ...
GL_FRONT_AND_BACK = ...
GL_FRONT_FACE = ...
GL_FUNC_ADD = ...
GL_FUNC_REVERSE_SUBTRACT = ...
GL_FUNC_SUBTRACT = ...
GL_GENERATE_MIPMAP_HINT = ...
GL_GEQUAL = ...
GL_GREATER = ...
GL_GREEN_BITS = ...
GL_HIGH_FLOAT = ...
GL_HIGH_INT = ...
GL_IMPLEMENTATION_COLOR_READ_FORMAT = ...
GL_IMPLEMENTATION_COLOR_READ_TYPE = ...
GL_INCR = ...
GL_INCR_WRAP = ...
GL_INFO_LOG_LENGTH = ...
GL_INT = ...
GL_INT_VEC2 = ...
GL_INT_VEC3 = ...
GL_INT_VEC4 = ...
GL_INVALID_ENUM = ...
GL_INVALID_FRAMEBUFFER_OPERATION = ...
GL_INVALID_OPERATION = ...
GL_INVALID_VALUE = ...
GL_INVERT = ...
GL_KEEP = ...
GL_LEQUAL = ...
GL_LESS = ...
GL_LINEAR = ...
GL_LINEAR_MIPMAP_LINEAR = ...
GL_LINEAR_MIPMAP_NEAREST = ...
GL_LINES = ...
GL_LINE_LOOP = ...
GL_LINE_STRIP = ...
GL_LINE_WIDTH = ...
GL_LINK_STATUS = ...
GL_LOW_FLOAT = ...
GL_LOW_INT = ...
GL_LUMINANCE = ...
GL_LUMINANCE_ALPHA = ...
GL_MAX = ...
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = ...
GL_MAX_CUBE_MAP_TEXTURE_SIZE = ...
GL_MAX_FRAGMENT_UNIFORM_VECTORS = ...
GL_MAX_RENDERBUFFER_SIZE = ...
GL_MAX_TEXTURE_IMAGE_UNITS = ...
GL_MAX_TEXTURE_SIZE = ...
GL_MAX_VARYING_VECTORS = ...
GL_MAX_VERTEX_ATTRIBS = ...
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = ...
GL_MAX_VERTEX_UNIFORM_VECTORS = ...
GL_MAX_VIEWPORT_DIMS = ...
GL_MEDIUM_FLOAT = ...
GL_MEDIUM_INT = ...
GL_MIN = ...
GL_MIRRORED_REPEAT = ...
GL_NEAREST = ...
GL_NEAREST_MIPMAP_LINEAR = ...
GL_NEAREST_MIPMAP_NEAREST = ...
GL_NEVER = ...
GL_NICEST = ...
GL_NONE = ...
GL_NOTEQUAL = ...
GL_NO_ERROR = ...
GL_NUM_COMPRESSED_TEXTURE_FORMATS = ...
GL_NUM_SHADER_BINARY_FORMATS = ...
GL_ONE = ...
GL_ONE_MINUS_CONSTANT_ALPHA = ...
GL_ONE_MINUS_CONSTANT_COLOR = ...
GL_ONE_MINUS_DST_ALPHA = ...
GL_ONE_MINUS_DST_COLOR = ...
GL_ONE_MINUS_SRC_ALPHA = ...
GL_ONE_MINUS_SRC_COLOR = ...
GL_OUT_OF_MEMORY = ...
GL_PACK_ALIGNMENT = ...
GL_POINTS = ...
GL_POLYGON_OFFSET_FACTOR = ...
GL_POLYGON_OFFSET_FILL = ...
GL_POLYGON_OFFSET_UNITS = ...
GL_RED_BITS = ...
GL_RENDERBUFFER = ...
GL_RENDERBUFFER_ALPHA_SIZE = ...
GL_RENDERBUFFER_BINDING = ...
GL_RENDERBUFFER_BLUE_SIZE = ...
GL_RENDERBUFFER_DEPTH_SIZE = ...
GL_RENDERBUFFER_GREEN_SIZE = ...
GL_RENDERBUFFER_HEIGHT = ...
GL_RENDERBUFFER_INTERNAL_FORMAT = ...
GL_RENDERBUFFER_RED_SIZE = ...
GL_RENDERBUFFER_STENCIL_SIZE = ...
GL_RENDERBUFFER_WIDTH = ...
GL_RENDERER = ...
GL_REPEAT = ...
GL_REPLACE = ...
GL_RGB = ...
GL_RGB565 = ...
GL_RGB5_A1 = ...
GL_RGBA = ...
GL_RGBA4 = ...
GL_SAMPLER_2D = ...
GL_SAMPLER_CUBE = ...
GL_SAMPLES = ...
GL_SAMPLE_ALPHA_TO_COVERAGE = ...
GL_SAMPLE_BUFFERS = ...
GL_SAMPLE_COVERAGE = ...
GL_SAMPLE_COVERAGE_INVERT = ...
GL_SAMPLE_COVERAGE_VALUE = ...
GL_SCISSOR_BOX = ...
GL_SCISSOR_TEST = ...
GL_SHADER_BINARY_FORMATS = ...
GL_SHADER_COMPILER = ...
GL_SHADER_SOURCE_LENGTH = ...
GL_SHADER_TYPE = ...
GL_SHADING_LANGUAGE_VERSION = ...
GL_SHORT = ...
GL_SRC_ALPHA = ...
GL_SRC_ALPHA_SATURATE = ...
GL_SRC_COLOR = ...
GL_STATIC_DRAW = ...
GL_STENCIL_ATTACHMENT = ...
GL_STENCIL_BACK_FAIL = ...
GL_STENCIL_BACK_FUNC = ...
GL_STENCIL_BACK_PASS_DEPTH_FAIL = ...
GL_STENCIL_BACK_PASS_DEPTH_PASS = ...
GL_STENCIL_BACK_REF = ...
GL_STENCIL_BACK_VALUE_MASK = ...
GL_STENCIL_BACK_WRITEMASK = ...
GL_STENCIL_BITS = ...
GL_STENCIL_BUFFER_BIT = ...
GL_STENCIL_CLEAR_VALUE = ...
GL_STENCIL_FAIL = ...
GL_STENCIL_FUNC = ...
GL_STENCIL_INDEX8 = ...
GL_STENCIL_PASS_DEPTH_FAIL = ...
GL_STENCIL_PASS_DEPTH_PASS = ...
GL_STENCIL_REF = ...
GL_STENCIL_TEST = ...
GL_STENCIL_VALUE_MASK = ...
GL_STENCIL_WRITEMASK = ...
GL_STREAM_DRAW = ...
GL_SUBPIXEL_BITS = ...
GL_TEXTURE = ...
GL_TEXTURE0 = ...
GL_TEXTURE1 = ...
GL_TEXTURE10 = ...
GL_TEXTURE11 = ...
GL_TEXTURE12 = ...
GL_TEXTURE13 = ...
GL_TEXTURE14 = ...
GL_TEXTURE15 = ...
GL_TEXTURE16 = ...
GL_TEXTURE17 = ...
GL_TEXTURE18 = ...
GL_TEXTURE19 = ...
GL_TEXTURE2 = ...
GL_TEXTURE20 = ...
GL_TEXTURE21 = ...
GL_TEXTURE22 = ...
GL_TEXTURE23 = ...
GL_TEXTURE24 = ...
GL_TEXTURE25 = ...
GL_TEXTURE26 = ...
GL_TEXTURE27 = ...
GL_TEXTURE28 = ...
GL_TEXTURE29 = ...
GL_TEXTURE3 = ...
GL_TEXTURE30 = ...
GL_TEXTURE31 = ...
GL_TEXTURE4 = ...
GL_TEXTURE5 = ...
GL_TEXTURE6 = ...
GL_TEXTURE7 = ...
GL_TEXTURE8 = ...
GL_TEXTURE9 = ...
GL_TEXTURE_2D = ...
GL_TEXTURE_BINDING_2D = ...
GL_TEXTURE_BINDING_CUBE_MAP = ...
GL_TEXTURE_CUBE_MAP = ...
GL_TEXTURE_CUBE_MAP_NEGATIVE_X = ...
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = ...
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = ...
GL_TEXTURE_CUBE_MAP_POSITIVE_X = ...
GL_TEXTURE_CUBE_MAP_POSITIVE_Y = ...
GL_TEXTURE_CUBE_MAP_POSITIVE_Z = ...
GL_TEXTURE_MAG_FILTER = ...
GL_TEXTURE_MIN_FILTER = ...
GL_TEXTURE_WRAP_S = ...
GL_TEXTURE_WRAP_T = ...
GL_TRIANGLES = ...
GL_TRIANGLE_FAN = ...
GL_TRIANGLE_STRIP = ...
GL_TRUE = ...
GL_UNPACK_ALIGNMENT = ...
GL_UNSIGNED_BYTE = ...
GL_UNSIGNED_INT = ...
GL_UNSIGNED_SHORT = ...
GL_UNSIGNED_SHORT_4_4_4_4 = ...
GL_UNSIGNED_SHORT_5_5_5_1 = ...
GL_UNSIGNED_SHORT_5_6_5 = ...
GL_VALIDATE_STATUS = ...
GL_VENDOR = ...
GL_VERSION = ...
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = ...
GL_VERTEX_ATTRIB_ARRAY_ENABLED = ...
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = ...
GL_VERTEX_ATTRIB_ARRAY_POINTER = ...
GL_VERTEX_ATTRIB_ARRAY_SIZE = ...
GL_VERTEX_ATTRIB_ARRAY_STRIDE = ...
GL_VERTEX_ATTRIB_ARRAY_TYPE = ...
GL_VERTEX_SHADER = ...
GL_VIEWPORT = ...
GL_ZERO = ...

ENUM_MAP: dict = ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

def isocurve(data: NDArray, level: float, connected: bool = False, extend_to_edge: bool = False): ...



from typing import ClassVar, Literal, TypeVar
from numpy.random import RandomState
from .base import BaseEstimator, ClassifierMixin, RegressorMixin, MultiOutputMixin
from .utils.validation import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
    check_is_fitted as check_is_fitted,
)
from .utils.multiclass import class_distribution as class_distribution
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from .utils import check_random_state as check_random_state
from .utils._param_validation import StrOptions as StrOptions, Interval as Interval
from ._typing import Int, ArrayLike, MatrixLike, Float

DummyRegressor_Self = TypeVar("DummyRegressor_Self", bound="DummyRegressor")
DummyClassifier_Self = TypeVar("DummyClassifier_Self", bound="DummyClassifier")

# Author: Mathieu Blondel <mathieu@mblondel.org>
#         Arnaud Joly <a.joly@ulg.ac.be>
#         Maheshakya Wijewardena <maheshakya.10@cse.mrt.ac.lk>
# License: BSD 3 clause

import warnings

import numpy as np
import scipy.sparse as sp


class DummyClassifier(MultiOutputMixin, ClassifierMixin, BaseEstimator):
    sparse_output_: bool = ...
    n_outputs_: int = ...
    class_prior_: ndarray | list[ArrayLike] = ...
    n_classes_: int | list[int] = ...
    classes_: ndarray | list[ArrayLike] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        strategy: Literal[
            "most_frequent", "prior", "stratified", "uniform", "constant", "prior"
        ] = "prior",
        random_state: RandomState | None | Int = None,
        constant: None | str | ArrayLike | int = None
    ) -> None:
        ...

    def fit(
        self: DummyClassifier_Self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> DummyClassifier_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray | list[ndarray]:
        ...

    def predict_log_proba(self, X: ArrayLike) -> ndarray | list[ndarray]:
        ...

    def score(
        self,
        X: None | MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> float:
        ...


class DummyRegressor(MultiOutputMixin, RegressorMixin, BaseEstimator):
    n_outputs_: int = ...
    constant_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        strategy: Literal["mean", "median", "quantile", "constant", "mean"] = "mean",
        constant: float | None | ArrayLike | int = None,
        quantile: float | None = None
    ) -> None:
        ...

    def fit(
        self: DummyRegressor_Self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> DummyRegressor_Self:
        ...

    def predict(
        self, X: MatrixLike, return_std: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def score(
        self,
        X: None | MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...



from numpy.typing import ArrayLike
import numpy as np

from .. import gloo
from .visual import Visual

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...

class InfiniteLineVisual(Visual):

    _shaders: dict = ...

    def __init__(
        self,
        pos: float | None = None,
        color: ArrayLike | tuple = ...,
        line_width: float = 1.0,
        antialias: bool = False,
        vertical=True,
        **kwargs,
    ): ...
    def set_data(self, pos: float | None = None, color: ArrayLike | tuple | None = None): ...
    @property
    def color(self): ...
    @property
    def pos(self): ...
    @property
    def line_width(self): ...
    @line_width.setter
    def line_width(self, val: float): ...
    @property
    def antialias(self): ...
    @antialias.setter
    def antialias(self, val: float): ...
    def _compute_bounds(self, axis, view): ...
    @property
    def is_vertical(self): ...
    def _prepare_transforms(self, view=None): ...
    def _prepare_draw(self, view=None): ...



from vispy.util.event import Event

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import math
import numpy as np

from ...util.quaternion import Quaternion
from ...util import keys
from .perspective import PerspectiveCamera

class FlyCamera(PerspectiveCamera):

    # Using _rotation1 and _rotation2 for camera states instead of _rotation
    _state_props = ...

    def __init__(self, fov: float = 60, rotation: None | float = None, **kwargs): ...
    @property
    def rotation(self): ...
    @rotation.setter
    def rotation(self, value): ...
    @property
    def rotation1(self): ...
    @rotation1.setter
    def rotation1(self, value): ...
    @property
    def rotation2(self): ...
    @rotation2.setter
    def rotation2(self, value): ...
    @property
    def auto_roll(self): ...
    @auto_roll.setter
    def auto_roll(self, value): ...
    @property
    def keymap(self): ...
    def _set_range(self, init): ...
    def _get_directions(self): ...
    def on_timer(self, event: Event): ...
    def viewbox_key_event(self, event: Event): ...
    def viewbox_mouse_event(self, event: Event): ...
    def _update_projection_transform(self, fx, fy): ...



from typing import ClassVar, Literal, TypeVar
from ..base import BaseEstimator
from numpy import ndarray
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from ..utils.metaestimators import available_if as available_if
from ..base import MetaEstimatorMixin, clone as clone
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import safe_mask as safe_mask
from ..utils.validation import check_is_fitted as check_is_fitted

SelfTrainingClassifier_Self = TypeVar(
    "SelfTrainingClassifier_Self", bound="SelfTrainingClassifier"
)

import warnings

import numpy as np

__all__ = ["SelfTrainingClassifier"]


class SelfTrainingClassifier(MetaEstimatorMixin, BaseEstimator):
    termination_condition_: Literal["max_iter", "no_change", "all_labeled"] = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    labeled_iter_: ndarray = ...
    transduction_: ndarray = ...
    classes_: ndarray | list[ndarray] = ...
    base_estimator_: BaseEstimator = ...

    _estimator_type: ClassVar[str] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        base_estimator: BaseEstimator,
        threshold: Float = 0.75,
        criterion: Literal["threshold", "k_best", "threshold"] = "threshold",
        k_best: Int = 10,
        max_iter: None | int = 10,
        verbose: bool = False,
    ) -> None:
        ...

    def fit(
        self: SelfTrainingClassifier_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> SelfTrainingClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score(self, X: MatrixLike | ArrayLike, y: ArrayLike) -> float:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._slsqp, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def slsqp(m, meq, x, xl, xu, f, c, g, a, acc, iter, mode, w, jw, alpha, f0, gs, h1, h2, h3, h4, t, t0, tol, iexact, incons, ireset, itermx, line, n1, n2, n3, la=..., n=..., l_w=..., l_jw=...) -> typing.Any:
    "slsqp(m,meq,x,xl,xu,f,c,g,a,acc,iter,mode,w,jw,alpha,f0,gs,h1,h2,h3,h4,t,t0,tol,iexact,incons,ireset,itermx,line,n1,n2,n3,[la,n,l_w,l_jw])\n\nWrapper for ``slsqp``.\n\nParameters\n----------\nm : input int\nmeq : input int\nx : in/output rank-1 array('d') with bounds (n)\nxl : input rank-1 array('d') with bounds (n)\nxu : input rank-1 array('d') with bounds (n)\nf : input float\nc : input rank-1 array('d') with bounds (la)\ng : input rank-1 array('d') with bounds (n + 1)\na : input rank-2 array('d') with bounds (la,n + 1)\nacc : in/output rank-0 array(float,'d')\niter : in/output rank-0 array(int,'i')\nmode : in/output rank-0 array(int,'i')\nw : input rank-1 array('d') with bounds (l_w)\njw : input rank-1 array('i') with bounds (l_jw)\nalpha : in/output rank-0 array(float,'d')\nf0 : in/output rank-0 array(float,'d')\ngs : in/output rank-0 array(float,'d')\nh1 : in/output rank-0 array(float,'d')\nh2 : in/output rank-0 array(float,'d')\nh3 : in/output rank-0 array(float,'d')\nh4 : in/output rank-0 array(float,'d')\nt : in/output rank-0 array(float,'d')\nt0 : in/output rank-0 array(float,'d')\ntol : in/output rank-0 array(float,'d')\niexact : in/output rank-0 array(int,'i')\nincons : in/output rank-0 array(int,'i')\nireset : in/output rank-0 array(int,'i')\nitermx : in/output rank-0 array(int,'i')\nline : in/output rank-0 array(int,'i')\nn1 : in/output rank-0 array(int,'i')\nn2 : in/output rank-0 array(int,'i')\nn3 : in/output rank-0 array(int,'i')\n\nOther Parameters\n----------------\nla : input int, optional\n    Default: len(c)\nn : input int, optional\n    Default: len(x)\nl_w : input int, optional\n    Default: len(w)\nl_jw : input int, optional\n    Default: len(jw)\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Callable, ClassVar, Iterable, TypeVar
from ..base import BaseEstimator
from ..model_selection._split import BaseShuffleSplit
from joblib import effective_n_jobs as effective_n_jobs
from ..metrics import check_scoring as check_scoring
from ..linear_model._logistic import LogisticRegression
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import check_is_fitted as check_is_fitted
from numpy import ndarray
from ..utils._param_validation import HasMethods as HasMethods, Interval as Interval
from numbers import Integral as Integral, Real as Real
from ..base import MetaEstimatorMixin, clone as clone, is_classifier as is_classifier
from ..model_selection import check_cv as check_cv
from ._base import SelectorMixin
from ..utils.metaestimators import available_if as available_if
from .._typing import Int, MatrixLike, ArrayLike
from ..model_selection import BaseCrossValidator

RFECV_Self = TypeVar("RFECV_Self", bound="RFECV")
RFE_Self = TypeVar("RFE_Self", bound="RFE")

# Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
#          Vincent Michel <vincent.michel@inria.fr>
#          Gilles Louppe <g.louppe@gmail.com>
#
# License: BSD 3 clause


import numpy as np


class RFE(SelectorMixin, MetaEstimatorMixin, BaseEstimator):
    support_: ndarray = ...
    ranking_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_features_: int = ...
    estimator_: BaseEstimator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: BaseEstimator,
        *,
        n_features_to_select: float | None | int = None,
        step: float | int = 1,
        verbose: Int = 0,
        importance_getter: str | Callable = "auto",
    ) -> None:
        ...

    @property
    def classes_(self) -> ndarray:
        ...

    def fit(
        self: RFE_Self, X: MatrixLike | ArrayLike, y: ArrayLike, **fit_params
    ) -> RFE_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def score(self, X: MatrixLike, y: ArrayLike, **fit_params) -> float:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...


class RFECV(RFE):
    support_: ndarray = ...
    ranking_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_features_: int = ...
    cv_results_: dict[str, ndarray] = ...
    estimator_: BaseEstimator = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: BaseEstimator | LogisticRegression,
        *,
        step: float | int = 1,
        min_features_to_select: Int = 1,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        scoring: None | str | Callable = None,
        verbose: Int = 0,
        n_jobs: None | int = None,
        importance_getter: str | Callable = "auto",
    ) -> None:
        ...

    def fit(
        self: RFECV_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        groups: None | ArrayLike = None,
    ) -> RFECV_Self:
        ...



"""
This type stub file was generated by pyright.
"""

import os
from traitlets.config.application import Application, catch_config_error
from traitlets.config.loader import PyFileConfigLoader

"""
An application for IPython.

All top-level applications should use the classes in this module for
handling configuration and creating configurables.

The job of an :class:`Application` is to create the master configuration
object and then create the configurable objects, passing the config to them.
"""
if os.name == "nt":
    programdata = ...
else:
    ...
ENV_CONFIG_DIRS = ...
_env_config_dir = ...
if _env_config_dir not in SYSTEM_CONFIG_DIRS:
    ...
_envvar = ...
if _envvar in None, '':
    IPYTHON_SUPPRESS_CONFIG_ERRORS = ...
else:
    ...
base_aliases = ...
if isinstance(Application.aliases, dict):
    ...
base_flags = ...
if isinstance(Application.flags, dict):
    ...
class ProfileAwareConfigLoader(PyFileConfigLoader):
    """A Python file config loader that is aware of IPython profiles."""
    def load_subconfig(self, fname, path=..., profile=...): # -> None:
        ...
    


class BaseIPythonApplication(Application):
    name = ...
    description = ...
    version = ...
    aliases = ...
    flags = ...
    classes = ...
    python_config_loader_class = ProfileAwareConfigLoader
    config_file_specified = ...
    config_file_name = ...
    builtin_profile_dir = ...
    config_file_paths = ...
    extra_config_file = ...
    profile = ...
    add_ipython_dir_to_sys_path = ...
    ipython_dir = ...
    _in_init_profile_dir = ...
    profile_dir = ...
    overwrite = ...
    auto_create = ...
    config_files = ...
    copy_config_files = ...
    verbose_crash = ...
    crash_handler_class = ...
    @catch_config_error
    def __init__(self, **kwargs) -> None:
        ...
    
    def init_crash_handler(self): # -> None:
        """Create a crash handler, typically setting sys.excepthook to it."""
        ...
    
    def excepthook(self, etype, evalue, tb): # -> Any | None:
        """this is sys.excepthook after init_crashhandler

        set self.verbose_crash=True to use our full crashhandler, instead of
        a regular traceback with a short message (crash_handler_lite)
        """
        ...
    
    def load_config_file(self, suppress_errors=...): # -> None:
        """Load the config file.

        By default, errors in loading config are handled, and a warning
        printed on screen. For testing, the suppress_errors option is set
        to False, so errors will make tests fail.

        `suppress_errors` default value is to be `None` in which case the
        behavior default to the one of `traitlets.Application`.

        The default value can be set :
           - to `False` by setting 'IPYTHON_SUPPRESS_CONFIG_ERRORS' environment variable to '0', or 'false' (case insensitive).
           - to `True` by setting 'IPYTHON_SUPPRESS_CONFIG_ERRORS' environment variable to '1' or 'true' (case insensitive).
           - to `None` by setting 'IPYTHON_SUPPRESS_CONFIG_ERRORS' environment variable to '' (empty string) or leaving it unset.

        Any other value are invalid, and will make IPython exit with a non-zero return code.
        """
        ...
    
    def init_profile_dir(self): # -> None:
        """initialize the profile dir"""
        ...
    
    def init_config_files(self): # -> None:
        """[optionally] copy default config files into profile dir."""
        ...
    
    def stage_default_config_file(self): # -> None:
        """auto generate default config file, and stage it into the profile."""
        ...
    
    @catch_config_error
    def initialize(self, argv=...): # -> None:
        ...
    






import numpy as np


def predict_stages(
    estimators: np.ndarray,
    X,
    scale: float,
    out: np.ndarray
) -> None:
    """Add predictions of ``estimators`` to ``out``.
    Each estimator is scaled by ``scale`` before its prediction
    is added to ``out``.
    """
    ...

def predict_stage(
    estimators: np.ndarray,
    stage: int,
    X,
    scale: float,
    out: np.ndarray
) -> None:
    """Add predictions of ``estimators[stage]`` to ``out``.
    Each estimator in the stage is scaled by ``scale`` before
    its prediction is added to ``out``.
    """
    ...



from typing import Any, Callable, Iterable, Optional, Type, TypeVar, Union

from django.utils.deprecation import MiddlewareMixin
from django.views.generic.base import View

_T = TypeVar("_T", bound=Union[View, Callable[..., Any]])  # Any callable

class classonlymethod(classmethod[Any, Any, Any]): ...

def method_decorator(
    decorator: Union[Callable[..., Any], Iterable[Callable[..., Any]]], name: str = ...
) -> Callable[[_T], _T]: ...
def decorator_from_middleware_with_args(
    middleware_class: type,
) -> Callable[..., Callable[[_T], _T]]: ...
def decorator_from_middleware(middleware_class: type) -> Callable[[_T], _T]: ...
def available_attrs(fn: Callable[..., Any]) -> Any: ...
def make_middleware_decorator(
    middleware_class: Type[MiddlewareMixin],
) -> Callable[..., Any]: ...

class classproperty:
    fget: Optional[Callable[..., Any]] = ...
    def __init__(self, method: Optional[Callable[..., Any]] = ...) -> None: ...
    def __get__(self, instance: Any, cls: Optional[type] = ...) -> Any: ...
    def getter(self, method: Callable[..., Any]) -> classproperty: ...

def sync_and_async_middleware(func: _T) -> _T: ...
def sync_only_middleware(func: _T) -> _T: ...
def async_only_middleware(func: _T) -> _T: ...



from ..utils import groups

class UnionFind:
    def __init__(self, elements=None): ...
    def __getitem__(self, object): ...
    def __iter__(self): ...
    def to_sets(self): ...
    def union(self, *objects): ...



from .scale import SymmetricalLogTransform
from typing import Any, Callable, Literal, Sequence
from ._typing import *
from .axis import Axis
from functools import partial

__all__ = (
    "TickHelper",
    "Formatter",
    "FixedFormatter",
    "NullFormatter",
    "FuncFormatter",
    "FormatStrFormatter",
    "StrMethodFormatter",
    "ScalarFormatter",
    "LogFormatter",
    "LogFormatterExponent",
    "LogFormatterMathtext",
    "LogFormatterSciNotation",
    "LogitFormatter",
    "EngFormatter",
    "PercentFormatter",
    "Locator",
    "IndexLocator",
    "FixedLocator",
    "NullLocator",
    "LinearLocator",
    "LogLocator",
    "AutoLocator",
    "MultipleLocator",
    "MaxNLocator",
    "AutoMinorLocator",
    "SymmetricalLogLocator",
    "AsinhLocator",
    "LogitLocator",
)

class _DummyAxis:

    dataLim = ...
    viewLim = ...
    def __init__(self, minpos: int = ...) -> None: ...
    def get_view_interval(self): ...
    def set_view_interval(self, vmin: float, vmax: float): ...
    def get_minpos(self): ...
    def get_data_interval(self): ...
    def set_data_interval(self, vmin: float, vmax: float): ...
    def get_tick_space(self): ...

class TickHelper:
    axis = ...
    def set_axis(self, axis: Axis) -> None: ...
    def create_dummy_axis(self, **kwargs) -> None: ...
    def set_view_interval(self, vmin: float, vmax: float): ...
    def set_data_interval(self, vmin: float, vmax: float): ...
    def set_bounds(self, vmin: float, vmax: float): ...

class Formatter(TickHelper):

    locs = ...
    def __call__(self, x, pos=...): ...
    def format_ticks(self, values: Sequence[float]) -> list[str]: ...
    def format_data(self, value) -> str: ...
    def format_data_short(self, value) -> str: ...
    def get_offset(self) -> str: ...
    def set_locs(self, locs: ArrayLike) -> None: ...
    @staticmethod
    def fix_minus(s: str) -> str: ...

class NullFormatter(Formatter):
    def __call__(self, x: float, pos: int = ...): ...

class FixedFormatter(Formatter):
    def __init__(self, seq: Sequence[str]) -> None: ...
    def __call__(self, x: float, pos: int = ...): ...
    def get_offset(self) -> str: ...
    def set_offset_string(self, ofs: str): ...

class FuncFormatter(Formatter):
    def __init__(self, func: Callable | partial) -> None: ...
    def __call__(self, x: float, pos: int = ...): ...
    def get_offset(self) -> str: ...
    def set_offset_string(self, ofs: str): ...

class FormatStrFormatter(Formatter):
    def __init__(self, fmt: str) -> None: ...
    def __call__(self, x: float, pos: int = ...): ...

class StrMethodFormatter(Formatter):
    def __init__(self, fmt: str) -> None: ...
    def __call__(self, x: float, pos: int = ...): ...

class ScalarFormatter(Formatter):
    def __init__(
        self,
        useOffset: bool | float = ...,
        useMathText: bool | None = ...,
        useLocale: bool | None = ...,
    ) -> None: ...
    def get_useOffset(self): ...
    def set_useOffset(self, val: bool | float) -> None: ...
    useOffset = ...
    def get_useLocale(self) -> bool | None: ...
    def set_useLocale(self, val: bool | None): ...
    useLocale = ...
    def get_useMathText(self) -> bool | None: ...
    def set_useMathText(self, val: bool | None) -> None: ...
    useMathText = ...
    def __call__(self, x: float, pos: int = ...) -> str: ...
    def set_scientific(self, bool) -> None: ...
    def set_powerlimits(self, lims: Sequence[int]): ...
    def format_data_short(self, value) -> str: ...
    def format_data(self, value) -> str: ...
    def get_offset(self) -> str: ...
    def set_locs(self, locs: Sequence[float]) -> None: ...

class LogFormatter(Formatter):
    def __init__(
        self,
        base: Sequence[float] = ...,
        labelOnlyBase: bool = False,
        minor_thresholds: Sequence[float] = ...,
        linthresh: None | float = None,
    ) -> None: ...
    def base(self, base): ...
    def label_minor(self, labelOnlyBase: bool): ...
    def set_locs(self, locs=...): ...
    def __call__(self, x: float, pos: int = ...): ...
    def format_data(self, value) -> str: ...
    def format_data_short(self, value) -> str: ...

class LogFormatterExponent(LogFormatter): ...

class LogFormatterMathtext(LogFormatter):
    def __call__(self, x: float, pos: int = ...): ...

class LogFormatterSciNotation(LogFormatterMathtext): ...

class LogitFormatter(Formatter):
    def __init__(
        self,
        *,
        use_overline: bool = False,
        one_half: str = "\\frac{1}{2}",
        minor: bool = False,
        minor_threshold: int = 25,
        minor_number: int = 6
    ) -> None: ...
    def use_overline(self, use_overline: bool = False): ...
    def set_one_half(self, one_half: str = "\\frac{1}{2}"): ...
    def set_minor_threshold(self, minor_threshold: int): ...
    def set_minor_number(self, minor_number: int): ...
    def set_locs(self, locs): ...
    def __call__(self, x: float, pos: int = ...): ...
    def format_data_short(self, value) -> str: ...

class EngFormatter(Formatter):

    ENG_PREFIXES = ...
    def __init__(
        self,
        unit: str = "",
        places: int | None = None,
        sep: str = " ",
        *,
        usetex: bool = ...,
        useMathText: bool = ...
    ) -> None: ...
    def get_usetex(self) -> bool: ...
    def set_usetex(self, val: bool) -> None: ...

    usetex = ...
    def get_useMathText(self) -> bool: ...
    def set_useMathText(self, val: bool) -> None: ...

    useMathText = ...
    def __call__(self, x: float, pos: int = ...): ...
    def format_eng(self, num) -> str: ...

class PercentFormatter(Formatter):
    def __init__(
        self,
        xmax: float = ...,
        decimals: None | int = ...,
        symbol: str | None = ...,
        is_latex: bool = ...,
    ) -> None: ...
    def __call__(self, x: float, pos: int = ...): ...
    def format_pct(self, x, display_range) -> str: ...
    def convert_to_pct(self, x): ...
    @property
    def symbol(self) -> str: ...
    @symbol.setter
    def symbol(self, symbol: str): ...

class Locator(TickHelper):

    MAXTICKS = ...
    def tick_values(self, vmin: float, vmax: float): ...
    def set_params(self, **kwargs): ...
    def __call__(self): ...
    def raise_if_exceeds(self, locs): ...
    def nonsingular(self, v0: float, v1: float) -> tuple[float, float]: ...
    def view_limits(self, vmin: float, vmax: float): ...

class IndexLocator(Locator):
    def __init__(self, base: float, offset: float) -> None: ...
    def set_params(self, base: float = ..., offset: float = ...): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...

class FixedLocator(Locator):
    def __init__(self, locs, nbins: int | None = ...) -> None: ...
    def set_params(self, nbins: int | None = ...): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...

class NullLocator(Locator):
    def __call__(self) -> list: ...
    def tick_values(self, vmin: float, vmax: float) -> list: ...

class LinearLocator(Locator):
    def __init__(self, numticks: int = ..., presets: dict = ...) -> None: ...
    @property
    def numticks(self): ...
    @numticks.setter
    def numticks(self, numticks: int): ...
    def set_params(self, numticks: int = ..., presets: dict = ...): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def view_limits(self, vmin: float, vmax: float): ...

class MultipleLocator(Locator):
    def __init__(self, base: float = ...) -> None: ...
    def set_params(self, base: float): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def view_limits(self, dmin: float, dmax: float): ...

def scale_range(
    vmin: float, vmax: float, n: int = ..., threshold: int = ...
) -> tuple[float, int]: ...

class _Edge_integer:
    def __init__(self, step: float, offset: float) -> None: ...
    def closeto(self, ms, edge): ...
    def le(self, x: float) -> float: ...
    def ge(self, x: float) -> float: ...

class MaxNLocator(Locator):

    default_params = ...
    def __init__(self, nbins: int | Literal["auto"] = 10, **kwargs) -> None: ...
    def set_params(self, **kwargs) -> None: ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def view_limits(self, dmin: float, dmax: float) -> tuple[float, float]: ...

def is_decade(x, base=..., *, rtol=...): ...
def is_close_to_int(x, *, atol=...): ...

class LogLocator(Locator):
    def __init__(
        self,
        base: float = ...,
        subs: None | str | Sequence[float] = ...,
        numdecs: int = ...,
        numticks: None | int = ...,
    ) -> None: ...
    def set_params(
        self,
        base: float = ...,
        subs: None | str | Sequence[float] = ...,
        numdecs: int = ...,
        numtick: None | int = ...,
    ): ...
    def base(self, base: float) -> None: ...
    def subs(self, subs: None | str | Sequence[float]) -> None: ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def view_limits(self, vmin: float, vmax: float): ...
    def nonsingular(self, vmin: float, vmax: float) -> tuple[float, float]: ...

class SymmetricalLogLocator(Locator):
    def __init__(
        self,
        transform: SymmetricalLogTransform = ...,
        subs: Sequence[float] = ...,
        linthresh: float = ...,
        base: float = ...,
    ) -> None: ...
    def set_params(self, subs: Sequence[float] = ..., numticks: int | None = ...): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def view_limits(self, vmin: float, vmax: float): ...

class AsinhLocator(Locator):
    def __init__(
        self,
        linear_width: float,
        numticks: int = 11,
        symthresh: float = 0.2,
        base: int = 10,
        subs: Sequence[int] = ...,
    ) -> None: ...
    def set_params(
        self,
        numticks: int = ...,
        symthresh: float = ...,
        base: int = ...,
        subs: Sequence[int] = ...,
    ): ...
    def __call__(self): ...
    def tick_values(self, vmin: float, vmax: float): ...

class LogitLocator(MaxNLocator):
    def __init__(
        self, minor: bool = ..., *, nbins: int | Literal["auto"] = ...
    ) -> None: ...
    def set_params(self, minor: None = ..., **kwargs) -> None: ...
    @property
    def minor(self): ...
    @minor.setter
    def minor(self, value): ...
    def tick_values(self, vmin: float, vmax: float): ...
    def nonsingular(self, vmin: float, vmax: float) -> tuple[float, float]: ...

class AutoLocator(MaxNLocator):
    def __init__(self) -> None: ...

class AutoMinorLocator(Locator):
    def __init__(self, n: int | None = ...) -> None: ...
    def __call__(self) -> list: ...
    def tick_values(self, vmin: float, vmax: float): ...



from numpy import ndarray
from numpy.typing import NDArray
from typing import Literal

import numpy as np

from .._shared.utils import check_nD

def graycomatrix(
    image,
    distances,
    angles,
    levels: int | None = None,
    symmetric: bool = False,
    normed: bool = False,
): ...
def graycoprops(
    P: NDArray,
    prop: Literal[
        "contrast", "dissimilarity", "homogeneity", "energy", "correlation", "ASM"
    ] = "contrast",
): ...
def local_binary_pattern(
    image,
    P: int,
    R: float,
    method: Literal["default", "ror", "uniform", "var"] = "default",
): ...
def multiblock_lbp(int_image, r: int, c: int, width: int, height: int) -> int: ...
def draw_multiblock_lbp(
    image,
    r: int,
    c: int,
    width: int,
    height: int,
    lbp_code: int = 0,
    color_greater_block: tuple[float, float, float] = ...,
    color_less_block=...,
    alpha: float = 0.5,
) -> np.ndarray: ...



from .font_manager import FontProperties
from typing import Literal, Sequence
from .transforms import Bbox, BboxBase, Transform
from .text import Text
from .backend_bases import Event, RendererBase
from .patches import Patch, Rectangle
from .lines import Line2D
from .figure import Figure
from .axes import Axes
from .artist import Artist, allow_rasterization
from .offsetbox import DraggableOffsetBox

class DraggableLegend(DraggableOffsetBox):
    def __init__(
        self, legend: Legend, use_blit: bool = ..., update: Literal["loc", "bbox"] = ...
    ) -> None: ...
    def finalize_offset(self): ...

class Legend(Artist):

    codes = ...
    zorder = ...
    def __str__(self) -> str: ...
    def __init__(
        self,
        parent: Axes | Figure,
        handles: Sequence[Artist],
        labels: Sequence[str],
        loc=...,
        numpoints=...,
        markerscale=...,
        markerfirst=...,
        scatterpoints=...,
        scatteryoffsets=...,
        prop=...,
        fontsize=...,
        labelcolor=...,
        borderpad=...,
        labelspacing=...,
        handlelength=...,
        handleheight=...,
        handletextpad=...,
        borderaxespad=...,
        columnspacing=...,
        ncols=...,
        mode=...,
        fancybox=...,
        shadow=...,
        title=...,
        title_fontsize=...,
        framealpha=...,
        edgecolor=...,
        facecolor=...,
        bbox_to_anchor=...,
        bbox_transform=...,
        frameon=...,
        handler_map=...,
        title_fontproperties=...,
        *,
        ncol=...
    ) -> None: ...
    def set_ncols(self, ncols: int) -> None: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    @classmethod
    def get_default_handler_map(cls): ...
    @classmethod
    def set_default_handler_map(cls, handler_map: dict): ...
    @classmethod
    def update_default_handler_map(cls, handler_map: dict): ...
    def get_legend_handler_map(self) -> dict: ...
    @staticmethod
    def get_legend_handler(legend_handler_map: dict, orig_handle): ...
    def get_children(self): ...
    def get_frame(self) -> Rectangle: ...
    def get_lines(self) -> list[Line2D]: ...
    def get_patches(self) -> list[Patch]: ...
    def get_texts(self) -> list[Text]: ...
    def set_title(self, title, prop: FontProperties = ...)-> None: ...
    def get_title(self) -> Text: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_tightbbox(self, renderer: RendererBase = ...) -> Bbox: ...
    def get_frame_on(self) -> bool: ...
    def set_frame_on(self, b: bool) -> None: ...
    draw_frame = ...
    def get_bbox_to_anchor(self) -> Bbox: ...
    def set_bbox_to_anchor(
        self, bbox: BboxBase | Sequence[float] | None, transform: Transform = ...
    ) -> None: ...
    def contains(self, event: Event) -> bool: ...
    def set_draggable(
        self, state: bool, use_blit: bool = ..., update: Literal["loc", "bbox"] = ...
    ) -> DraggableLegend: ...
    def get_draggable(self) -> bool: ...



from typing import (
    Callable,
    Sequence,
)

class ExcelCell:
    __fields__ = ...
    row = ...
    col = ...
    val = ...
    style = ...
    mergestart = ...
    mergeend = ...
    def __init__(
        self, row: int, col: int, val, style=..., mergestart=..., mergeend=...
    ) -> None: ...

class CSSToExcelConverter:
    inherited = ...
    def __init__(self, inherited: str | None = ...) -> None: ...
    compute_css = ...
    def __call__(self, declarations_str: str) -> dict[str, dict[str, str]]: ...
    def build_xlstyle(self, props: dict[str, str]) -> dict[str, dict[str, str]]: ...
    VERTICAL_MAP = ...
    def build_alignment(self, props) -> dict[str, bool | str | None]: ...
    def build_border(self, props: dict) -> dict[str, dict[str, str]]: ...
    def build_fill(self, props: dict[str, str]): ...
    BOLD_MAP = ...
    ITALIC_MAP = ...
    def build_font(self, props) -> dict[str, bool | int | str | None]: ...
    NAMED_COLORS = ...
    def color_to_excel(self, val: str | None): ...
    def build_number_format(self, props: dict) -> dict[str, str | None]: ...

class ExcelFormatter:
    max_rows = ...
    max_cols = ...
    rowcounter: int = ...
    na_rep = ...
    styler = ...
    style_converter = ...
    df = ...
    columns = ...
    float_format = ...
    index = ...
    index_label = ...
    header = ...
    merge_cells = ...
    inf_rep = ...
    def __init__(
        self,
        df,
        na_rep: str = ...,
        float_format: str | None = ...,
        cols: Sequence | None = ...,
        header: bool | list[str] = ...,
        index: bool = ...,
        index_label: str | Sequence | None = ...,
        merge_cells: bool = ...,
        inf_rep: str = ...,
        style_converter: Callable | None = ...,
    ) -> None: ...
    @property
    def header_style(self): ...
    def get_formatted_cells(self) -> None: ...
    def write(
        self,
        writer,
        sheet_name: str = ...,
        startrow: int = ...,
        startcol: int = ...,
        freeze_panes=...,
        engine=...,
    ) -> None: ...



from typing import Callable
from networkx import Graph, DiGraph
from ..classes.graph import Graph
from ..classes.coreviews import (
    FilterAdjacency,
    FilterAtlas,
    FilterMultiAdjacency,
    UnionAdjacency,
    UnionMultiAdjacency,
)
from ..classes.filters import no_filter
from ..exception import NetworkXError
from ..utils import not_implemented_for

__all__ = ["generic_graph_view", "subgraph_view", "reverse_view"]

def generic_graph_view(G: Graph, create_using=None): ...
def subgraph_view(
    G: Graph, filter_node: Callable = ..., filter_edge: Callable = ...
) -> Graph: ...
def reverse_view(G: DiGraph) -> DiGraph: ...



"""
This type stub file was generated by pyright.
"""

from typing import Any, Dict, List, Optional, Tuple, Union
from .basic import Booster
from .sklearn import LGBMModel

"""Plotting library."""
def plot_importance(booster: Union[Booster, LGBMModel], ax=..., height: float = ..., xlim: Optional[Tuple[float, float]] = ..., ylim: Optional[Tuple[float, float]] = ..., title: Optional[str] = ..., xlabel: Optional[str] = ..., ylabel: Optional[str] = ..., importance_type: str = ..., max_num_features: Optional[int] = ..., ignore_zero: bool = ..., figsize: Optional[Tuple[float, float]] = ..., dpi: Optional[int] = ..., grid: bool = ..., precision: Optional[int] = ..., **kwargs: Any) -> Any:
    """Plot model's feature importances.

    Parameters
    ----------
    booster : Booster or LGBMModel
        Booster or LGBMModel instance which feature importance should be plotted.
    ax : matplotlib.axes.Axes or None, optional (default=None)
        Target axes instance.
        If None, new figure and axes will be created.
    height : float, optional (default=0.2)
        Bar height, passed to ``ax.barh()``.
    xlim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.xlim()``.
    ylim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.ylim()``.
    title : str or None, optional (default="Feature importance")
        Axes title.
        If None, title is disabled.
    xlabel : str or None, optional (default="Feature importance")
        X-axis title label.
        If None, title is disabled.
        @importance_type@ placeholder can be used, and it will be replaced with the value of ``importance_type`` parameter.
    ylabel : str or None, optional (default="Features")
        Y-axis title label.
        If None, title is disabled.
    importance_type : str, optional (default="auto")
        How the importance is calculated.
        If "auto", if ``booster`` parameter is LGBMModel, ``booster.importance_type`` attribute is used; "split" otherwise.
        If "split", result contains numbers of times the feature is used in a model.
        If "gain", result contains total gains of splits which use the feature.
    max_num_features : int or None, optional (default=None)
        Max number of top features displayed on plot.
        If None or <1, all features will be displayed.
    ignore_zero : bool, optional (default=True)
        Whether to ignore features with zero importance.
    figsize : tuple of 2 elements or None, optional (default=None)
        Figure size.
    dpi : int or None, optional (default=None)
        Resolution of the figure.
    grid : bool, optional (default=True)
        Whether to add a grid for axes.
    precision : int or None, optional (default=3)
        Used to restrict the display of floating point values to a certain precision.
    **kwargs
        Other parameters passed to ``ax.barh()``.

    Returns
    -------
    ax : matplotlib.axes.Axes
        The plot with model's feature importances.
    """
    ...

def plot_split_value_histogram(booster: Union[Booster, LGBMModel], feature: Union[int, str], bins: Union[int, str, None] = ..., ax=..., width_coef: float = ..., xlim: Optional[Tuple[float, float]] = ..., ylim: Optional[Tuple[float, float]] = ..., title: Optional[str] = ..., xlabel: Optional[str] = ..., ylabel: Optional[str] = ..., figsize: Optional[Tuple[float, float]] = ..., dpi: Optional[int] = ..., grid: bool = ..., **kwargs: Any) -> Any:
    """Plot split value histogram for the specified feature of the model.

    Parameters
    ----------
    booster : Booster or LGBMModel
        Booster or LGBMModel instance of which feature split value histogram should be plotted.
    feature : int or str
        The feature name or index the histogram is plotted for.
        If int, interpreted as index.
        If str, interpreted as name.
    bins : int, str or None, optional (default=None)
        The maximum number of bins.
        If None, the number of bins equals number of unique split values.
        If str, it should be one from the list of the supported values by ``numpy.histogram()`` function.
    ax : matplotlib.axes.Axes or None, optional (default=None)
        Target axes instance.
        If None, new figure and axes will be created.
    width_coef : float, optional (default=0.8)
        Coefficient for histogram bar width.
    xlim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.xlim()``.
    ylim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.ylim()``.
    title : str or None, optional (default="Split value histogram for feature with @index/name@ @feature@")
        Axes title.
        If None, title is disabled.
        @feature@ placeholder can be used, and it will be replaced with the value of ``feature`` parameter.
        @index/name@ placeholder can be used,
        and it will be replaced with ``index`` word in case of ``int`` type ``feature`` parameter
        or ``name`` word in case of ``str`` type ``feature`` parameter.
    xlabel : str or None, optional (default="Feature split value")
        X-axis title label.
        If None, title is disabled.
    ylabel : str or None, optional (default="Count")
        Y-axis title label.
        If None, title is disabled.
    figsize : tuple of 2 elements or None, optional (default=None)
        Figure size.
    dpi : int or None, optional (default=None)
        Resolution of the figure.
    grid : bool, optional (default=True)
        Whether to add a grid for axes.
    **kwargs
        Other parameters passed to ``ax.bar()``.

    Returns
    -------
    ax : matplotlib.axes.Axes
        The plot with specified model's feature split value histogram.
    """
    ...

def plot_metric(booster: Union[Dict, LGBMModel], metric: Optional[str] = ..., dataset_names: Optional[List[str]] = ..., ax=..., xlim: Optional[Tuple[float, float]] = ..., ylim: Optional[Tuple[float, float]] = ..., title: Optional[str] = ..., xlabel: Optional[str] = ..., ylabel: Optional[str] = ..., figsize: Optional[Tuple[float, float]] = ..., dpi: Optional[int] = ..., grid: bool = ...) -> Any:
    """Plot one metric during training.

    Parameters
    ----------
    booster : dict or LGBMModel
        Dictionary returned from ``lightgbm.train()`` or LGBMModel instance.
    metric : str or None, optional (default=None)
        The metric name to plot.
        Only one metric supported because different metrics have various scales.
        If None, first metric picked from dictionary (according to hashcode).
    dataset_names : list of str, or None, optional (default=None)
        List of the dataset names which are used to calculate metric to plot.
        If None, all datasets are used.
    ax : matplotlib.axes.Axes or None, optional (default=None)
        Target axes instance.
        If None, new figure and axes will be created.
    xlim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.xlim()``.
    ylim : tuple of 2 elements or None, optional (default=None)
        Tuple passed to ``ax.ylim()``.
    title : str or None, optional (default="Metric during training")
        Axes title.
        If None, title is disabled.
    xlabel : str or None, optional (default="Iterations")
        X-axis title label.
        If None, title is disabled.
    ylabel : str or None, optional (default="@metric@")
        Y-axis title label.
        If 'auto', metric name is used.
        If None, title is disabled.
        @metric@ placeholder can be used, and it will be replaced with metric name.
    figsize : tuple of 2 elements or None, optional (default=None)
        Figure size.
    dpi : int or None, optional (default=None)
        Resolution of the figure.
    grid : bool, optional (default=True)
        Whether to add a grid for axes.

    Returns
    -------
    ax : matplotlib.axes.Axes
        The plot with metric's history over the training.
    """
    ...

def create_tree_digraph(booster: Union[Booster, LGBMModel], tree_index: int = ..., show_info: Optional[List[str]] = ..., precision: Optional[int] = ..., orientation: str = ..., **kwargs: Any) -> Any:
    """Create a digraph representation of specified tree.

    Each node in the graph represents a node in the tree.

    Non-leaf nodes have labels like ``Column_10 <= 875.9``, which means
    "this node splits on the feature named "Column_10", with threshold 875.9".

    Leaf nodes have labels like ``leaf 2: 0.422``, which means "this node is a
    leaf node, and the predicted value for records that fall into this node
    is 0.422". The number (``2``) is an internal unique identifier and doesn't
    have any special meaning.

    .. note::

        For more information please visit
        https://graphviz.readthedocs.io/en/stable/api.html#digraph.

    Parameters
    ----------
    booster : Booster or LGBMModel
        Booster or LGBMModel instance to be converted.
    tree_index : int, optional (default=0)
        The index of a target tree to convert.
    show_info : list of str, or None, optional (default=None)
        What information should be shown in nodes.

            - ``'split_gain'`` : gain from adding this split to the model
            - ``'internal_value'`` : raw predicted value that would be produced by this node if it was a leaf node
            - ``'internal_count'`` : number of records from the training data that fall into this non-leaf node
            - ``'internal_weight'`` : total weight of all nodes that fall into this non-leaf node
            - ``'leaf_count'`` : number of records from the training data that fall into this leaf node
            - ``'leaf_weight'`` : total weight (sum of hessian) of all observations that fall into this leaf node
            - ``'data_percentage'`` : percentage of training data that fall into this node
    precision : int or None, optional (default=3)
        Used to restrict the display of floating point values to a certain precision.
    orientation : str, optional (default='horizontal')
        Orientation of the tree.
        Can be 'horizontal' or 'vertical'.
    **kwargs
        Other parameters passed to ``Digraph`` constructor.
        Check https://graphviz.readthedocs.io/en/stable/api.html#digraph for the full list of supported parameters.

    Returns
    -------
    graph : graphviz.Digraph
        The digraph representation of specified tree.
    """
    ...

def plot_tree(booster: Union[Booster, LGBMModel], ax=..., tree_index: int = ..., figsize: Optional[Tuple[float, float]] = ..., dpi: Optional[int] = ..., show_info: Optional[List[str]] = ..., precision: Optional[int] = ..., orientation: str = ..., **kwargs: Any) -> Any:
    """Plot specified tree.

    Each node in the graph represents a node in the tree.

    Non-leaf nodes have labels like ``Column_10 <= 875.9``, which means
    "this node splits on the feature named "Column_10", with threshold 875.9".

    Leaf nodes have labels like ``leaf 2: 0.422``, which means "this node is a
    leaf node, and the predicted value for records that fall into this node
    is 0.422". The number (``2``) is an internal unique identifier and doesn't
    have any special meaning.

    .. note::

        It is preferable to use ``create_tree_digraph()`` because of its lossless quality
        and returned objects can be also rendered and displayed directly inside a Jupyter notebook.

    Parameters
    ----------
    booster : Booster or LGBMModel
        Booster or LGBMModel instance to be plotted.
    ax : matplotlib.axes.Axes or None, optional (default=None)
        Target axes instance.
        If None, new figure and axes will be created.
    tree_index : int, optional (default=0)
        The index of a target tree to plot.
    figsize : tuple of 2 elements or None, optional (default=None)
        Figure size.
    dpi : int or None, optional (default=None)
        Resolution of the figure.
    show_info : list of str, or None, optional (default=None)
        What information should be shown in nodes.

            - ``'split_gain'`` : gain from adding this split to the model
            - ``'internal_value'`` : raw predicted value that would be produced by this node if it was a leaf node
            - ``'internal_count'`` : number of records from the training data that fall into this non-leaf node
            - ``'internal_weight'`` : total weight of all nodes that fall into this non-leaf node
            - ``'leaf_count'`` : number of records from the training data that fall into this leaf node
            - ``'leaf_weight'`` : total weight (sum of hessian) of all observations that fall into this leaf node
            - ``'data_percentage'`` : percentage of training data that fall into this node
    precision : int or None, optional (default=3)
        Used to restrict the display of floating point values to a certain precision.
    orientation : str, optional (default='horizontal')
        Orientation of the tree.
        Can be 'horizontal' or 'vertical'.
    **kwargs
        Other parameters passed to ``Digraph`` constructor.
        Check https://graphviz.readthedocs.io/en/stable/api.html#digraph for the full list of supported parameters.

    Returns
    -------
    ax : matplotlib.axes.Axes
        The plot with single tree.
    """
    ...




from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from numpy.random import RandomState

from ..util import _straight_line_vertices

def random(
    adjacency_mat: ArrayLike | NDArray,
    directed: bool = False,
    random_state: None | int | RandomState = None,
): ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""
A context manager for managing things injected into :mod:`builtins`.
"""
class __BuiltinUndefined:
    ...


BuiltinUndefined = ...
class __HideBuiltin:
    ...


HideBuiltin = ...
class BuiltinTrap(Configurable):
    shell = ...
    def __init__(self, shell=...) -> None:
        ...
    
    def __enter__(self): # -> Self@BuiltinTrap:
        ...
    
    def __exit__(self, type, value, traceback):
        ...
    
    def add_builtin(self, key, value): # -> None:
        """Add a builtin and save the original."""
        ...
    
    def remove_builtin(self, key, orig): # -> None:
        """Remove an added builtin and re-set the original."""
        ...
    
    def activate(self): # -> None:
        """Store ipython references in the __builtin__ namespace."""
        ...
    
    def deactivate(self): # -> None:
        """Remove any builtins which might have been added by add_builtins, or
        restore overwritten ones to their previous values."""
        ...
    





from typing import ClassVar, Iterable, Literal, TypeVar
from .model_selection import (
    check_cv as check_cv,
    cross_val_predict as cross_val_predict,
)
from .isotonic import IsotonicRegression
from .utils.validation import (
    check_consistent_length as check_consistent_length,
    check_is_fitted as check_is_fitted,
)
from .base import BaseEstimator
from .model_selection._split import BaseShuffleSplit
from scipy.optimize import fmin_bfgs as fmin_bfgs
from inspect import signature as signature
from scipy.special import expit as expit, xlogy as xlogy
from .utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from matplotlib.artist import Artist
from .base import (
    ClassifierMixin,
    RegressorMixin,
    clone as clone,
    MetaEstimatorMixin,
    is_classifier as is_classifier,
)
from numpy import ndarray
from matplotlib.axes import Axes
from numbers import Integral as Integral
from .svm import LinearSVC as LinearSVC
from functools import partial as partial
from .utils._param_validation import (
    StrOptions as StrOptions,
    HasMethods as HasMethods,
    Hidden as Hidden,
)
from ._typing import Int, MatrixLike, ArrayLike
from matplotlib.figure import Figure
from .preprocessing import (
    label_binarize as label_binarize,
    LabelEncoder as LabelEncoder,
)
from .utils import (
    column_or_1d as column_or_1d,
    indexable as indexable,
    check_matplotlib_support as check_matplotlib_support,
)
from math import log as log
from .utils.parallel import delayed as delayed, Parallel as Parallel
from .model_selection import BaseCrossValidator

CalibratedClassifierCV_Self = TypeVar(
    "CalibratedClassifierCV_Self", bound="CalibratedClassifierCV"
)
_SigmoidCalibration_Self = TypeVar(
    "_SigmoidCalibration_Self", bound="_SigmoidCalibration"
)

import warnings
import numpy as np


class CalibratedClassifierCV(ClassifierMixin, MetaEstimatorMixin, BaseEstimator):
    calibrated_classifiers_: list = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: None | BaseEstimator = None,
        *,
        method: Literal["sigmoid", "isotonic", "sigmoid"] = "sigmoid",
        cv: int | BaseCrossValidator | Iterable | None | str | BaseShuffleSplit = None,
        n_jobs: None | Int = None,
        ensemble: bool = True,
        base_estimator: str | BaseEstimator = "deprecated",
    ) -> None:
        ...

    def fit(
        self: CalibratedClassifierCV_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
        **fit_params,
    ) -> CalibratedClassifierCV_Self:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...


class _CalibratedClassifier:
    def __init__(
        self,
        estimator: BaseEstimator,
        calibrators: list[IsotonicRegression | _SigmoidCalibration]
        | list[BaseEstimator],
        *,
        classes: ArrayLike,
        method: Literal["sigmoid", "isotonic", "sigmoid"] = "sigmoid",
    ) -> None:
        ...

    def predict_proba(self, X: ArrayLike) -> ndarray:
        ...


class _SigmoidCalibration(RegressorMixin, BaseEstimator):
    b_: float = ...
    a_: float = ...

    def fit(
        self: _SigmoidCalibration_Self,
        X: ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> _SigmoidCalibration_Self:
        ...

    def predict(self, T: ArrayLike) -> ndarray:
        ...


def calibration_curve(
    y_true: ArrayLike,
    y_prob: ArrayLike,
    *,
    pos_label: None | str | Int = None,
    normalize: str | bool = "deprecated",
    n_bins: Int = 5,
    strategy: Literal["uniform", "quantile", "uniform"] = "uniform",
) -> tuple[ndarray, ndarray]:
    ...


class CalibrationDisplay:
    figure_: Figure = ...
    ax_: Axes = ...
    line_: Artist = ...

    def __init__(
        self,
        prob_true: ArrayLike,
        prob_pred: ArrayLike,
        y_prob: ArrayLike,
        *,
        estimator_name: None | str = None,
        pos_label: None | str | int = None,
    ) -> None:
        ...

    def plot(
        self,
        *,
        ax: None | Axes = None,
        name: None | str = None,
        ref_line: bool = True,
        **kwargs,
    ) -> CalibrationDisplay:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator: BaseEstimator,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        *,
        n_bins: Int = 5,
        strategy: Literal["uniform", "quantile", "uniform"] = "uniform",
        pos_label: None | str | int = None,
        name: None | str = None,
        ref_line: bool = True,
        ax: None | Axes = None,
        **kwargs,
    ) -> CalibrationDisplay:
        ...

    @classmethod
    def from_predictions(
        cls,
        y_true: ArrayLike,
        y_prob: ArrayLike,
        *,
        n_bins: Int = 5,
        strategy: Literal["uniform", "quantile", "uniform"] = "uniform",
        pos_label: None | str | int = None,
        name: None | str = None,
        ref_line: bool = True,
        ax: None | Axes = None,
        **kwargs,
    ) -> CalibrationDisplay:
        ...



from vispy.util.event import Event

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .panzoom import PanZoomCamera
from ...visuals.transforms.nonlinear import MagnifyTransform, Magnify1DTransform

class MagnifyCamera(PanZoomCamera):

    transform_class = ...

    def __init__(self, size_factor: float = 0.25, radius_ratio: float = 0.9, **kwargs): ...
    def _viewbox_set(self, viewbox): ...
    def _viewbox_unset(self, viewbox): ...
    def viewbox_mouse_event(self, event: Event): ...
    def on_timer(self, event: Event | None = None): ...
    def viewbox_resize_event(self, event: Event): ...
    def view_changed(self): ...

class Magnify1DCamera(MagnifyCamera):
    transform_class = ...
    __doc__ = ...




from typing import Literal
import numpy as np
from numpy.random import RandomState

def sample_without_replacement(n_population: int, n_samples: int, random_state: int|RandomState|None=None, method: Literal["auto", "tracking_selection", "reservoir_sampling", "pool"] = "auto") -> np.ndarray: ...



"""
This type stub file was generated by pyright.
"""

"""Support for interactive macros in IPython"""
coding_declaration = ...
class Macro:
    """Simple class to store the value of macros as strings.

    Macro is just a callable that executes a string of IPython
    input when called.
    """
    def __init__(self, code) -> None:
        """store the macro value, as a single string which can be executed"""
        ...
    
    def __str__(self) -> str:
        ...
    
    def __repr__(self):
        ...
    
    def __getstate__(self):
        """ needed for safe pickling via %store """
        ...
    
    def __add__(self, other): # -> Macro:
        ...
    





__all__ = ["network_simplex"]

from itertools import chain, islice, repeat
from math import ceil, sqrt

from ...classes.graph import Graph
from ...utils import not_implemented_for

class _DataEssentialsAndFunctions:
    def __init__(
        self,
        G: Graph,
        multigraph,
        demand="demand",
        capacity="capacity",
        weight="weight",
    ): ...
    def initialize_spanning_tree(self, n, faux_inf): ...
    def find_apex(self, p, q): ...
    def trace_path(self, p, w): ...
    def find_cycle(self, i, p, q): ...
    def augment_flow(self, Wn, We, f): ...
    def trace_subtree(self, p): ...
    def remove_edge(self, s, t): ...
    def make_root(self, q): ...
    def add_edge(self, i, p, q): ...
    def update_potentials(self, i, p, q): ...
    def reduced_cost(self, i): ...
    def find_entering_edges(self): ...
    def residual_capacity(self, i, p): ...
    def find_leaving_edge(self, Wn, We): ...

def network_simplex(
    G: Graph, demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
): ...



"""
This type stub file was generated by pyright.
"""

"""Top-level display functions for displaying object in different formats."""
__all__ = ['display', 'clear_output', 'publish_display_data', 'update_display', 'DisplayHandle']
class _Sentinel:
    def __repr__(self):
        ...
    


_sentinel = ...
def publish_display_data(data, metadata=..., source=..., *, transient=..., **kwargs): # -> None:
    """Publish data and metadata to all frontends.

    See the ``display_data`` message in the messaging documentation for
    more details about this message type.

    Keys of data and metadata can be any mime-type.

    Parameters
    ----------
    data : dict
        A dictionary having keys that are valid MIME types (like
        'text/plain' or 'image/svg+xml') and values that are the data for
        that MIME type. The data itself must be a JSON'able data
        structure. Minimally all data should have the 'text/plain' data,
        which can be displayed by all frontends. If more than the plain
        text is given, it is up to the frontend to decide which
        representation to use.
    metadata : dict
        A dictionary for metadata related to the data. This can contain
        arbitrary key, value pairs that frontends can use to interpret
        the data. mime-type keys matching those in data can be used
        to specify metadata about particular representations.
    source : str, deprecated
        Unused.
    transient : dict, keyword-only
        A dictionary of transient data, such as display_id.
    """
    ...

def display(*objs, include=..., exclude=..., metadata=..., transient=..., display_id=..., raw=..., clear=..., **kwargs): # -> DisplayHandle | None:
    """Display a Python object in all frontends.

    By default all representations will be computed and sent to the frontends.
    Frontends can decide which representation is used and how.

    In terminal IPython this will be similar to using :func:`print`, for use in richer
    frontends see Jupyter notebook examples with rich display logic.

    Parameters
    ----------
    *objs : object
        The Python objects to display.
    raw : bool, optional
        Are the objects to be displayed already mimetype-keyed dicts of raw display data,
        or Python objects that need to be formatted before display? [default: False]
    include : list, tuple or set, optional
        A list of format type strings (MIME types) to include in the
        format data dict. If this is set *only* the format types included
        in this list will be computed.
    exclude : list, tuple or set, optional
        A list of format type strings (MIME types) to exclude in the format
        data dict. If this is set all format types will be computed,
        except for those included in this argument.
    metadata : dict, optional
        A dictionary of metadata to associate with the output.
        mime-type keys in this dictionary will be associated with the individual
        representation formats, if they exist.
    transient : dict, optional
        A dictionary of transient data to associate with the output.
        Data in this dict should not be persisted to files (e.g. notebooks).
    display_id : str, bool optional
        Set an id for the display.
        This id can be used for updating this display area later via update_display.
        If given as `True`, generate a new `display_id`
    clear : bool, optional
        Should the output area be cleared before displaying anything? If True,
        this will wait for additional output before clearing. [default: False]
    **kwargs : additional keyword-args, optional
        Additional keyword-arguments are passed through to the display publisher.

    Returns
    -------
    handle: DisplayHandle
        Returns a handle on updatable displays for use with :func:`update_display`,
        if `display_id` is given. Returns :any:`None` if no `display_id` is given
        (default).

    Examples
    --------
    >>> class Json(object):
    ...     def __init__(self, json):
    ...         self.json = json
    ...     def _repr_pretty_(self, pp, cycle):
    ...         import json
    ...         pp.text(json.dumps(self.json, indent=2))
    ...     def __repr__(self):
    ...         return str(self.json)
    ...

    >>> d = Json({1:2, 3: {4:5}})

    >>> print(d)
    {1: 2, 3: {4: 5}}

    >>> display(d)
    {
      "1": 2,
      "3": {
        "4": 5
      }
    }

    >>> def int_formatter(integer, pp, cycle):
    ...     pp.text('I'*integer)

    >>> plain = get_ipython().display_formatter.formatters['text/plain']
    >>> plain.for_type(int, int_formatter)
    <function _repr_pprint at 0x...>
    >>> display(7-5)
    II

    >>> del plain.type_printers[int]
    >>> display(7-5)
    2

    See Also
    --------
    :func:`update_display`

    Notes
    -----
    In Python, objects can declare their textual representation using the
    `__repr__` method. IPython expands on this idea and allows objects to declare
    other, rich representations including:

      - HTML
      - JSON
      - PNG
      - JPEG
      - SVG
      - LaTeX

    A single object can declare some or all of these representations; all are
    handled by IPython's display system.

    The main idea of the first approach is that you have to implement special
    display methods when you define your class, one for each representation you
    want to use. Here is a list of the names of the special methods and the
    values they must return:

      - `_repr_html_`: return raw HTML as a string, or a tuple (see below).
      - `_repr_json_`: return a JSONable dict, or a tuple (see below).
      - `_repr_jpeg_`: return raw JPEG data, or a tuple (see below).
      - `_repr_png_`: return raw PNG data, or a tuple (see below).
      - `_repr_svg_`: return raw SVG data as a string, or a tuple (see below).
      - `_repr_latex_`: return LaTeX commands in a string surrounded by "$",
                        or a tuple (see below).
      - `_repr_mimebundle_`: return a full mimebundle containing the mapping
                             from all mimetypes to data.
                             Use this for any mime-type not listed above.

    The above functions may also return the object's metadata alonside the
    data.  If the metadata is available, the functions will return a tuple
    containing the data and metadata, in that order.  If there is no metadata
    available, then the functions will return the data only.

    When you are directly writing your own classes, you can adapt them for
    display in IPython by following the above approach. But in practice, you
    often need to work with existing classes that you can't easily modify.

    You can refer to the documentation on integrating with the display system in
    order to register custom formatters for already existing types
    (:ref:`integrating_rich_display`).

    .. versionadded:: 5.4 display available without import
    .. versionadded:: 6.1 display available without import

    Since IPython 5.4 and 6.1 :func:`display` is automatically made available to
    the user without import. If you are using display in a document that might
    be used in a pure python context or with older version of IPython, use the
    following import at the top of your file::

        from IPython.display import display

    """
    ...

def update_display(obj, *, display_id, **kwargs): # -> None:
    """Update an existing display by id

    Parameters
    ----------
    obj
        The object with which to update the display
    display_id : keyword-only
        The id of the display to update

    See Also
    --------
    :func:`display`
    """
    ...

class DisplayHandle:
    """A handle on an updatable display

    Call `.update(obj)` to display a new object.

    Call `.display(obj`) to add a new instance of this display,
    and update existing instances.

    See Also
    --------

        :func:`display`, :func:`update_display`

    """
    def __init__(self, display_id=...) -> None:
        ...
    
    def __repr__(self):
        ...
    
    def display(self, obj, **kwargs): # -> None:
        """Make a new display with my id, updating existing instances.

        Parameters
        ----------
        obj
            object to display
        **kwargs
            additional keyword arguments passed to display
        """
        ...
    
    def update(self, obj, **kwargs): # -> None:
        """Update existing displays with my id

        Parameters
        ----------
        obj
            object to display
        **kwargs
            additional keyword arguments passed to update_display
        """
        ...
    


def clear_output(wait=...): # -> None:
    """Clear the output of the current cell receiving output.

    Parameters
    ----------
    wait : bool [default: false]
        Wait to clear the output until new output is available to replace it."""
    ...




from ...utils import not_implemented_for
from ..matching import maximal_matching
from ...classes.graph import Graph

__all__ = ["min_weighted_dominating_set", "min_edge_dominating_set"]

# TODO Why doesn't this algorithm work for directed graphs?

def min_weighted_dominating_set(G: Graph, weight: str | None = None) -> set: ...
def min_edge_dominating_set(G: Graph) -> set: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
__all__ = [
    "ArcballCamera",
    "BaseCamera",
    "FlyCamera",
    "MagnifyCamera",
    "Magnify1DCamera",
    "PanZoomCamera",
    "TurntableCamera",
]

from ._base import make_camera as make_camera  # noqa
from .base_camera import BaseCamera as BaseCamera  # noqa
from .panzoom import PanZoomCamera as PanZoomCamera  # noqa
from .arcball import ArcballCamera as ArcballCamera  # noqa
from .turntable import TurntableCamera as TurntableCamera  # noqa
from .fly import FlyCamera as FlyCamera  # noqa
from .magnify import (
    MagnifyCamera as MagnifyCamera,
    Magnify1DCamera as Magnify1DCamera,
)  # noqa



from collections import OrderedDict
from typing import Any, Iterator, List, Optional, Tuple, Union

from django.core.files.storage import FileSystemStorage

def matches_patterns(
    path: str, patterns: Union[List[str], Tuple[str], OrderedDict[Any, Any]] = ...
) -> bool: ...
def get_files(
    storage: FileSystemStorage, ignore_patterns: List[str] = ..., location: str = ...
) -> Iterator[str]: ...
def check_settings(base_url: Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

class DashAtlas(object):
    def __init__(self, shape=...): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def make_pattern(self, pattern, caps=[1, 1]): ...



from typing import Mapping
from scipy import sparse as sparse
from .._typing import ArrayLike, MatrixLike
from numpy import ndarray
from ..preprocessing import LabelEncoder as LabelEncoder

# Authors: Andreas Mueller
#          Manoj Kumar
# License: BSD 3 clause

import numpy as np


def compute_class_weight(
    class_weight: dict[int, int] | None | Mapping | str,
    *,
    classes: ArrayLike,
    y: ArrayLike
) -> ndarray:
    ...


def compute_sample_weight(
    class_weight: None | Mapping | str | ArrayLike,
    y: MatrixLike | ArrayLike,
    *,
    indices: None | ArrayLike = None
) -> ndarray:
    ...



latex_to_bakoma: dict[str, tuple] = ...
type12uni: dict[str, int] = ...
uni2type1: dict[str, int] = ...
tex2uni: dict[str, int] = ...
stix_virtual_fonts: dict[str, dict[str, list[tuple]]] = ...
stix_glyph_fixes: dict[int, int] = ...



from ..classes.digraph import DiGraph
from numpy.typing import ArrayLike
from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "laplacian_matrix",
    "normalized_laplacian_matrix",
    "total_spanning_tree_weight",
    "directed_laplacian_matrix",
    "directed_combinatorial_laplacian_matrix",
]

def laplacian_matrix(G: Graph, nodelist: ArrayLike | None = None, weight="weight"): ...
def normalized_laplacian_matrix(
    G: Graph, nodelist: ArrayLike | None = None, weight="weight"
): ...
def total_spanning_tree_weight(G: Graph, weight: str | None = None) -> float: ...

###############################################################################
# Code based on work from https://github.com/bjedwards

def directed_laplacian_matrix(
    G: DiGraph,
    nodelist: ArrayLike | None = None,
    weight="weight",
    walk_type=None,
    alpha=0.95,
): ...
def directed_combinatorial_laplacian_matrix(
    G: DiGraph,
    nodelist: ArrayLike | None = None,
    weight="weight",
    walk_type=None,
    alpha=0.95,
): ...



from matplotlib.path import Path
import numpy as np
from typing import Literal, Sequence, Type
from matplotlib._typing import *
from matplotlib.transforms import Affine2DBase, ScaledTranslation, Transform
from matplotlib.text import Text
from matplotlib.backend_bases import MouseButton
from matplotlib.lines import Line2D
from matplotlib.transforms import Bbox
from matplotlib.axes import Axes
from matplotlib.axis import Axis, XTick, YTick
from matplotlib.ticker import Formatter, Locator

class PolarTransform(Transform):

    input_dims = ...
    def __init__(self, axis=..., use_rmin=..., _apply_theta_transforms=...) -> None: ...
    def transform_non_affine(self, tr) -> np.ndarray: ...
    def transform_path_non_affine(self, path: Path)-> Path: ...
    def inverted(self)-> InvertedPolarTransform: ...

class PolarAffine(Affine2DBase):
    def __init__(self, scale_transform, limits) -> None: ...
    def get_matrix(self) -> np.ndarray|None: ...

class InvertedPolarTransform(Transform):
    input_dims = ...
    def __init__(self, axis=..., use_rmin=..., _apply_theta_transforms=...) -> None: ...
    def transform_non_affine(self, xy) -> np.ndarray: ...
    def inverted(self)-> PolarTransform: ...

class ThetaFormatter(Formatter):
    def __call__(self, x, pos=...): ...

class _AxisWrapper:
    def __init__(self, axis: Axis) -> None: ...
    def get_view_interval(self): ...
    def set_view_interval(self, vmin: float, vmax: float)-> None: ...
    def get_minpos(self): ...
    def get_data_interval(self): ...
    def set_data_interval(self, vmin: float, vmax: float)-> None: ...
    def get_tick_space(self): ...

class ThetaLocator(Locator):
    def __init__(self, base) -> None: ...
    def set_axis(self, axis: Axis)-> None: ...
    def __call__(self): ...
    def refresh(self): ...
    def view_limits(self, vmin: float, vmax: float): ...

class ThetaTick(XTick):
    def __init__(self, axes, *args, **kwargs) -> None: ...
    def update_position(self, loc)-> None: ...

class ThetaAxis(Axis):
    axis_name: str = ...
    _tick_class: Type[ThetaTick] = ThetaTick
    def clear(self)-> None: ...

class RadialLocator(Locator):
    def __init__(self, base, axes: Axes = ...) -> None: ...
    def set_axis(self, axis: Axis)-> None: ...
    def __call__(self): ...
    def nonsingular(self, vmin: float, vmax: float)-> tuple[float, float]: ...
    def view_limits(self, vmin: float, vmax: float): ...

class _ThetaShift(ScaledTranslation):
    def __init__(
        self, axes: Axes, pad: float, mode: Literal["min", "max", "rlabel"]
    ) -> None: ...
    def get_matrix(self)-> np.ndarray|None: ...

class RadialTick(YTick):
    def __init__(self, *args, **kwargs) -> None: ...
    def update_position(self, loc)-> None: ...

class RadialAxis(Axis):
    axis_name: str = ...
    _tick_class: Type[RadialTick] = RadialTick
    def __init__(self, *args, **kwargs) -> None: ...
    def clear(self)-> None: ...

class _WedgeBbox(Bbox):
    def __init__(
        self, center: Sequence[float], viewLim: Bbox, originLim: Bbox, **kwargs
    ) -> None: ...
    def get_points(self)-> np.ndarray: ...

class PolarAxes(Axes):
    name = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def clear(self)-> None: ...
    def get_xaxis_transform(self, which=...)-> Transform: ...
    def get_xaxis_text1_transform(self, pad)-> Transform: ...
    def get_xaxis_text2_transform(self, pad)-> Transform: ...
    def get_yaxis_transform(self, which=...)-> Transform: ...
    def get_yaxis_text1_transform(self, pad)-> Transform: ...
    def get_yaxis_text2_transform(self, pad)-> Transform: ...
    def draw(self, renderer)-> None: ...
    def set_thetamax(self, thetamax)-> None: ...
    def get_thetamax(self)-> np.ndarray: ...
    def set_thetamin(self, thetamin)-> None: ...
    def get_thetamin(self)-> np.ndarray: ...
    def set_thetalim(self, *args, **kwargs)-> None: ...
    def set_theta_offset(self, offset): ...
    def get_theta_offset(self)->np.ndarray: ...
    def set_theta_zero_location(self, loc: str, offset: float = 0)-> None: ...
    def set_theta_direction(self, direction: int)-> None: ...
    def get_theta_direction(self) -> int: ...
    def set_rmax(self, rmax: float)-> None: ...
    def get_rmax(self) -> float: ...
    def set_rmin(self, rmin: float)-> None: ...
    def get_rmin(self) -> float: ...
    def set_rorigin(self, rorigin: float)-> None: ...
    def get_rorigin(self) -> float: ...
    def get_rsign(self)-> np.ndarray: ...
    def set_rlim(self, bottom=..., top=..., emit=..., auto=..., **kwargs)-> None: ...
    def get_rlabel_position(self) -> float: ...
    def set_rlabel_position(self, value: float)-> None: ...
    def set_yscale(self, *args, **kwargs)-> None: ...
    def set_rscale(self, *args, **kwargs)-> None: ...
    def set_rticks(self, *args, **kwargs)-> None: ...
    def set_thetagrids(
        self,
        angles: tuple[float, ...],
        labels: tuple[str, ...] | None = ...,
        fmt: str | None = ...,
        **kwargs
    ) -> tuple[list[Line2D], list[Text]]: ...
    def set_rgrids(
        self,
        radii: tuple[float, ...],
        labels: None | tuple[str, ...] = ...,
        angle: float = ...,
        fmt: str | None = ...,
        **kwargs
    ) -> tuple[list[Line2D], list[Text]]: ...
    def format_coord(self, theta, r)-> str: ...
    def get_data_ratio(self) -> float: ...
    def can_zoom(self) -> bool: ...
    def can_pan(self) -> bool: ...
    def start_pan(self, x: float, y: float, button: MouseButton)-> None: ...
    def end_pan(self)-> None: ...
    def drag_pan(self, button: MouseButton, key: str | None, x: float, y: float)-> None: ...



from typing import Iterable, Sequence
from ._typing import *
from .ticker import Formatter, Locator
from .axis import Axis
from datetime import date

class ConversionError(TypeError): ...

class AxisInfo:
    def __init__(
        self,
        majloc: Locator = ...,
        minloc: Locator = ...,
        majfmt: Formatter = ...,
        minfmt: Formatter = ...,
        label: str | None = ...,
        default_limits: Sequence[date] = ...,
    ) -> None: ...

class ConversionInterface:
    @staticmethod
    def axisinfo(unit, axis: Axis) -> AxisInfo: ...
    @staticmethod
    def default_units(x, axis: Axis): ...
    @staticmethod
    def convert(obj, unit, axis: Axis): ...
    @staticmethod
    def is_numlike(x: str) -> bool: ...

class DecimalConverter(ConversionInterface):
    @staticmethod
    def convert(value: Decimal | Iterable, unit, axis: Axis): ...

class Registry(dict):
    def get_converter(self, x) -> ConversionInterface | None: ...

registry: dict = ...



from pandas.plotting._matplotlib.core import LinePlot as LinePlot

class HistPlot(LinePlot):
    bins = ...
    bottom = ...
    def __init__(self, data, bins: int = ..., bottom: int = ..., **kwargs) -> None: ...
    @property
    def orientation(self): ...

class KdePlot(HistPlot):
    orientation: str = ...
    bw_method = ...
    ind = ...
    def __init__(self, data, bw_method=..., ind=..., **kwargs) -> None: ...

def hist_series(
    self,
    by=...,
    ax=...,
    grid: bool = ...,
    xlabelsize=...,
    xrot=...,
    ylabelsize=...,
    yrot=...,
    figsize=...,
    bins: int = ...,
    **kwds,
): ...
def hist_frame(
    data,
    column=...,
    by=...,
    grid: bool = ...,
    xlabelsize=...,
    xrot=...,
    ylabelsize=...,
    yrot=...,
    ax=...,
    sharex: bool = ...,
    sharey: bool = ...,
    figsize=...,
    layout=...,
    bins: int = ...,
    **kwds,
): ...



__all__ = ["generate_adjlist", "write_adjlist", "parse_adjlist", "read_adjlist"]

from ..classes.graph import Graph
from ..utils import open_file

def generate_adjlist(G: Graph, delimiter: str = " ") -> str: ...
def write_adjlist(
    G: Graph, path, comments: str = "#", delimiter: str = " ", encoding: str = "utf-8"
): ...
def parse_adjlist(
    lines,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
): ...
def read_adjlist(
    path,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    encoding="utf-8",
): ...



from typing import (
    Any,
    Callable,
    ClassVar,
    Collection,
    Dict,
    Iterable,
    List,
    Optional,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from django.core.checks.messages import CheckMessage
from django.core.exceptions import (
    MultipleObjectsReturned as BaseMultipleObjectsReturned,
)
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db.models.manager import BaseManager
from django.db.models.options import Options

_M = TypeVar("_M", bound=Any)
_Self = TypeVar("_Self", bound="Model")

class ModelStateFieldsCacheDescriptor: ...

class ModelState:
    db: Optional[str] = ...
    adding: bool = ...
    fields_cache: ModelStateFieldsCacheDescriptor = ...

class ModelBase(type):
    # FIXME: It would be better to use _Self instead of _M here,
    # but pyright says Type[_Self] cannot be assigned here... Maybe a bug in pyright?
    @property
    def objects(cls: Type[_M]) -> BaseManager[_M]: ...
    @property
    def _meta(cls: Type[_M]) -> Options[_M]: ...
    @property
    def _default_manager(cls: Type[_M]) -> BaseManager[_M]: ...

class Model(metaclass=ModelBase):
    DoesNotExist: ClassVar[type[ObjectDoesNotExist]]
    MultipleObjectsReturned: ClassVar[type[BaseMultipleObjectsReturned]]

    class Meta: ...
    pk: Any = ...
    _state: ModelState
    _meta: Options[Any]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @classmethod
    def add_to_class(cls, name: str, value: Any) -> Any: ...
    @classmethod
    def from_db(
        cls: Type[_Self],
        db: Optional[str],
        field_names: Collection[str],
        values: Collection[Any],
    ) -> _Self: ...
    def delete(
        self, using: Any = ..., keep_parents: bool = ...
    ) -> Tuple[int, Dict[str, int]]: ...
    async def adelete(
        self, using: Any = ..., keep_parents: bool = ...
    ) -> Tuple[int, Dict[str, int]]: ...
    def full_clean(
        self, exclude: Optional[Collection[str]] = ..., validate_unique: bool = True, validate_constraints: bool = True
    ) -> None: ...
    def clean(self) -> None: ...
    def clean_fields(self, exclude: Optional[Collection[str]] = ...) -> None: ...
    def validate_unique(self, exclude: Optional[Collection[str]] = ...) -> None: ...
    def unique_error_message(
        self: _Self,
        model_class: Type[_Self],
        unique_check: Collection[Union[Callable[..., Any], str]],
    ) -> ValidationError: ...
    def save(
        self,
        force_insert: bool = ...,
        force_update: bool = ...,
        using: Optional[str] = ...,
        update_fields: Optional[Iterable[str]] = ...,
    ) -> None: ...
    async def asave(
        self,
        force_insert: bool = ...,
        force_update: bool = ...,
        using: Optional[str] = ...,
        update_fields: Optional[Iterable[str]] = ...,
    ) -> None: ...
    def save_base(
        self,
        raw: bool = ...,
        force_insert: bool = ...,
        force_update: bool = ...,
        using: Optional[str] = ...,
        update_fields: Optional[Iterable[str]] = ...,
    ) -> Any: ...
    def refresh_from_db(
        self, using: Optional[str] = ..., fields: Optional[List[str]] = ...
    ) -> None: ...
    async def arefresh_from_db(
        self, using: Optional[str] = ..., fields: Optional[List[str]] = ...
    ) -> None: ...
    def get_deferred_fields(self) -> Set[str]: ...
    @classmethod
    def check(cls, **kwargs: Any) -> List[CheckMessage]: ...
    def __getstate__(self) -> Dict[Any, Any]: ...



from numpy.typing import NDArray

from .mesh import MeshVisual
from ..geometry import create_grid_mesh, MeshData

class GridMeshVisual(MeshVisual):
    def __init__(
        self, xs: NDArray, ys: NDArray, zs: NDArray, colors: NDArray | None = None, shading: str | None = "smooth", **kwargs
    ): ...
    def set_data(
        self,
        xs: NDArray | None = None,
        ys: NDArray | None = None,
        zs: NDArray | None = None,
        colors: NDArray | None = None,
    ): ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config import Configurable

"""
Color managing related utilities
"""
available_themes = ...
class Colorable(Configurable):
    """
    A subclass of configurable for all the classes that have a `default_scheme`
    """
    default_style = ...





from ..gloo.buffer import VertexBuffer
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import re
import numpy as np

from .globject import GLObject
from .buffer import VertexBuffer, IndexBuffer, DataBuffer
from .texture import BaseTexture, Texture2D, Texture3D, Texture1D, TextureCube
from ..util import logger
from .util import check_enum
from .context import get_current_canvas
from .preprocessor import preprocess

# ------------------------------------------------------------ Constants ---
REGEX_VAR: dict = ...

# ------------------------------------------------------------ Shader class ---
class Shader(GLObject):
    def __init__(self, code=None): ...
    @property
    def code(self): ...
    @code.setter
    def code(self, code): ...

class VertexShader(Shader):
    _GLIR_TYPE: str = ...

class FragmentShader(Shader):
    _GLIR_TYPE: str = ...

class GeometryShader(Shader):
    _GLIR_TYPE: str = ...

# ----------------------------------------------------------- Program class ---
class Program(GLObject):

    _GLIR_TYPE: str = ...

    _gtypes: dict = ...

    # ---------------------------------
    def __init__(self, vert: str | None = None, frag: str | None = None, count: int | None = 0): ...
    def set_shaders(
        self,
        vert: str,
        frag: str,
        geom: str | None = None,
        update_variables: bool = True,
    ): ...
    @property
    def shaders(self): ...
    @property
    def variables(self) -> ArrayLike: ...
    def _parse_variables_from_code(self, update_variables=True): ...
    def bind(self, data: VertexBuffer): ...
    def _process_pending_variables(self): ...
    def __setitem__(self, name, data): ...
    def __contains__(self, key): ...
    def __getitem__(self, name): ...
    def draw(
        self,
        mode: str = "triangles",
        indices: ArrayLike | None = None,
        check_error=True,
    ): ...



from ...classes.graph import Graph

__all__ = ["is_arborescence", "is_branching", "is_forest", "is_tree"]

def is_arborescence(G: Graph) -> bool: ...
def is_branching(G: Graph) -> bool: ...
def is_forest(G: Graph) -> bool: ...
def is_tree(G: Graph) -> bool: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division as division

__all__ = ["use", "sys_info", "set_log_level", "test"]

from .util import (
    config as config,
    set_log_level as set_log_level,
    keys as keys,
    sys_info as sys_info,
)  # noqa
from .util.wrappers import use as use, test as test  # noqa

def _get_sg_image_scraper(): ...



from functools import partial
from itertools import chain

from ..classes.graph import Graph
from ..utils import arbitrary_element, not_implemented_for

__all__ = ["min_edge_cover", "is_edge_cover"]

def min_edge_cover(G: Graph, matching_algorithm=None) -> set: ...
def is_edge_cover(G: Graph, cover: set) -> bool: ...



from vispy.util.svg.color import Color

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..scene import SceneCanvas
from .plotwidget import PlotWidget

class Fig(SceneCanvas):
    def __init__(self, bgcolor: Color | str = "w", size: tuple = ..., show: bool = True, keys="interactive", **kwargs): ...
    @property
    def plot_widgets(self): ...
    def __getitem__(self, idxs): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import time as systime

# get a reference starting time - initial performance counter
START_TIME = ...

def time(): ...



"""
This type stub file was generated by pyright.
"""

"""Usage information for the main IPython applications.
"""
cl_usage = ...
interactive_usage = ...
interactive_usage_min = ...
quick_reference = ...
default_banner_parts = ...
default_banner = ...



import math

import numpy as np
import scipy.ndimage as ndi

from .._shared.utils import check_nD, _supported_float_type
from ..feature.util import DescriptorExtractor, FeatureDetector
from .._shared.filters import gaussian
from ..transform import rescale

def _edgeness(hxx, hyy, hxy): ...
def _sparse_gradient(vol, positions): ...
def _hessian(d, positions): ...
def _offsets(grad, hess): ...

class SIFT(FeatureDetector, DescriptorExtractor):
    def __init__(
        self,
        upsampling: int = 2,
        n_octaves: int = 8,
        n_scales: int = 3,
        sigma_min: float = 1.6,
        sigma_in: float = 0.5,
        c_dog: float = ...,
        c_edge: float = 10,
        n_bins: int = 36,
        lambda_ori: float = 1.5,
        c_max: float = 0.8,
        lambda_descr: float = 6,
        n_hist: int = 4,
        n_ori: int = 8,
    ): ...
    @property
    def deltas(self): ...
    def _set_number_of_octaves(self, image_shape): ...
    def _create_scalespace(self, image): ...
    def _inrange(self, a, dim): ...
    def _find_localize_evaluate(self, dogspace, img_shape): ...
    def _fit(self, h): ...
    def _compute_orientation(
        self, positions_oct, scales_oct, sigmas_oct, octaves, gaussian_scalespace
    ): ...
    def _rotate(self, row, col, angle): ...
    def _compute_descriptor(self, gradient_space): ...
    def _preprocess(self, image): ...
    def detect(self, image): ...
    def extract(self, image): ...
    def detect_and_extract(self, image): ...



from typing import Any, Callable, Generator, Iterable, Type

import functools
from .deprecation import (
    MatplotlibDeprecationWarning,
)

class classproperty:
    def __init__(
        self, fget: Callable, fset: None = ..., fdel: None = ..., doc: None = ...
    ) -> None: ...
    def __get__(self, instance, owner): ...
    @property
    def fget(self): ...

def check_isinstance(**kwargs) -> None: ...
def check_in_list(
    _values: Iterable, *, _print_supported_values: bool = True, **kwargs
) -> None: ...
def check_shape(**kwargs) -> None: ...
def check_getitem(_mapping: dict, **kwargs): ...
def caching_module_getattr(cls) -> functools._lru_cache_wrapper: ...
def define_aliases(
    alias_d: dict[str, list[str]], cls: None = ...
) -> functools.partial: ...
def select_matching_signature(funcs: list[Callable], *args, **kwargs): ...
def recursive_subclasses(cls) -> Generator: ...
def warn_external(
    message: MatplotlibDeprecationWarning | PendingDeprecationWarning | str,
    category: None | Type[MatplotlibDeprecationWarning] = ...,
) -> None: ...



def radd(left, right): ...
def rsub(left, right): ...
def rmul(left, right): ...
def rdiv(left, right): ...
def rtruediv(left, right): ...
def rfloordiv(left, right): ...
def rmod(left, right): ...
def rdivmod(left, right): ...
def rpow(left, right): ...
def rand_(left, right): ...
def ror_(left, right): ...
def rxor(left, right): ...



from matplotlib.widgets import LockDraw
from typing import Callable
from .backend_tools import ToolBase
from .figure import Figure
from .backend_bases import Event, FigureCanvasBase

class ToolEvent:
    def __init__(self, name: str, sender, tool, data=...) -> None: ...

class ToolTriggerEvent(ToolEvent):
    def __init__(self, name: str, sender, tool, canvasevent=..., data=...) -> None: ...

class ToolManagerMessageEvent:
    def __init__(self, name: str, sender, message) -> None: ...

class ToolManager:

    keypresslock: LockDraw
    messagelock: LockDraw

    def __init__(self, figure=...) -> None: ...
    @property
    def canvas(self) -> FigureCanvasBase: ...
    @property
    def figure(self) -> Figure: ...
    @figure.setter
    def figure(self, figure: Figure) -> None: ...
    def set_figure(self, figure: Figure, update_tools: bool = ...)-> None: ...
    def toolmanager_connect(self, s: str, func: Callable) -> int: ...
    def toolmanager_disconnect(self, cid: int): ...
    def message_event(self, message, sender=...)-> None: ...
    @property
    def active_toggle(self)-> dict: ...
    def get_tool_keymap(self, name: str) -> list[str]: ...
    def update_keymap(self, name: str, key: list[str] | str) -> None: ...
    def remove_tool(self, name: str) -> None: ...
    def add_tool(self, name: str, tool: type, *args, **kwargs)-> ToolBase: ...
    def trigger_tool(
        self,
        name: str,
        sender: object = ...,
        canvasevent: Event = ...,
        data: object = ...,
    )-> None: ...
    @property
    def tools(self)-> dict[str, ToolBase]: ...
    def get_tool(
        self, name: str | ToolBase, warn: bool = ...
    ) -> tuple[ToolBase, None]: ...



from numpy.typing import NDArray
from typing import Callable
import functools

import numpy as np
from scipy import ndimage as ndi

from .._shared.utils import deprecate_kwarg

from .misc import default_footprint

__all__ = ["erosion", "dilation", "opening", "closing", "white_tophat", "black_tophat"]

def _shift_footprint(footprint, shift_x, shift_y): ...
def _invert_footprint(footprint): ...
def pad_for_eccentric_footprints(func: Callable) -> Callable: ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def erosion(
    image: NDArray,
    footprint: NDArray | None = None,
    out=None,
    shift_x: bool = False,
    shift_y: bool = False,
): ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def dilation(
    image: NDArray,
    footprint: NDArray | None = None,
    out: NDArray | None = None,
    shift_x: bool = False,
    shift_y: bool = False,
): ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
@default_footprint
@pad_for_eccentric_footprints
def opening(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
@default_footprint
@pad_for_eccentric_footprints
def closing(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def white_tophat(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def black_tophat(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...



from typing import (
    Any,
    Callable,
    Iterable,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

from django.core.files.base import File
from django.core.files.images import ImageFile
from django.core.files.storage import FileSystemStorage, Storage
from django.db.models.base import Model
from django.db.models.fields import (
    _GT,
    Field,
    _ErrorMessagesToOverride,
    _ValidatorCallable,
)

class FieldFile(File):
    instance: Model = ...
    field: FileField = ...
    storage: FileSystemStorage = ...
    def __init__(
        self, instance: Model, field: FileField, name: Optional[str]
    ) -> None: ...
    file: Any = ...
    @property
    def path(self) -> str: ...
    @property
    def url(self) -> str: ...
    @property
    def size(self) -> int: ...
    def save(self, name: str, content: File, save: bool = ...) -> None: ...
    def delete(self, save: bool = ...) -> None: ...
    @property
    def closed(self) -> bool: ...

class FileDescriptor:
    field: FileField = ...
    def __init__(self, field: FileField) -> None: ...
    def __set__(self, instance: Model, value: Optional[Any]) -> None: ...
    def __get__(
        self, instance: Optional[Model], cls: Type[Model] = ...
    ) -> Union[FieldFile, FileDescriptor]: ...

_T = TypeVar("_T", bound="Field[Any, Any]")

class FileField(Field[FileDescriptor, FileDescriptor]):
    storage: Any = ...
    upload_to: Union[str, Callable[[Any, str], str]] = ...
    def __new__(
        cls,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        upload_to: Union[str, Callable[[Any, str], str]] = ...,
        storage: Optional[Union[Storage, Callable[[], Storage]]] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Optional[Union[_GT, Callable[[], _GT]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_GT, str], Tuple[str, Iterable[Tuple[_GT, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> FileField: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner: Any) -> FileDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> FieldFile: ...
    # non-Model instances
    @overload
    def __get__(self: _T, instance: Any, owner: Any) -> _T: ...
    def generate_filename(self, instance: Optional[Model], filename: str) -> str: ...

class ImageFileDescriptor(FileDescriptor):
    field: ImageField
    def __set__(self, instance: Model, value: Optional[str]) -> None: ...

class ImageFieldFile(ImageFile, FieldFile):
    field: ImageField
    def delete(self, save: bool = ...) -> None: ...

class ImageField(FileField):
    def __new__(
        cls,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        width_field: str = ...,
        height_field: str = ...,
        upload_to: Union[str, Callable[[Model, str], Any]] = ...,
        storage: Optional[Union[Storage, Callable[[], Storage]]] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Optional[Union[_GT, Callable[[], _GT]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[_GT, str], Tuple[str, Iterable[Tuple[_GT, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> ImageField: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner: Any) -> ImageFileDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> ImageFieldFile: ...
    # non-Model instances
    @overload
    def __get__(self: _T, instance: Any, owner: Any) -> _T: ...
    def update_dimension_fields(
        self, instance: Model, force: bool = ..., *args: Any, **kwargs: Any
    ) -> None: ...



from numpy.typing import ArrayLike
from typing import Sequence

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .ellipse import EllipseVisual

class RegularPolygonVisual(EllipseVisual):
    def __init__(
        self,
        center: ArrayLike | None = None,
        color: str | tuple | Sequence = "black",
        border_color: str | tuple | Sequence | None = None,
        border_width: float = 1,
        radius: float = 0.1,
        sides: int = 4,
        **kwargs,
    ): ...
    @property
    def sides(self): ...
    @sides.setter
    def sides(self, sides): ...



import logging
from argparse import ArgumentParser
from io import StringIO
from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Type
from unittest import TestCase as _TestCase
from unittest import TestSuite, TextTestResult

from django.db.backends.base.base import BaseDatabaseWrapper
from django.test.testcases import SimpleTestCase, TestCase
from django.utils.datastructures import OrderedSet

class DebugSQLTextTestResult(TextTestResult):
    buffer: bool
    descriptions: bool
    dots: bool
    expectedFailures: List[Any]
    failfast: bool
    shouldStop: bool
    showAll: bool
    skipped: List[Any]
    tb_locals: bool
    testsRun: int
    unexpectedSuccesses: List[Any]
    logger: logging.Logger = ...
    def __init__(self, stream: Any, descriptions: bool, verbosity: int) -> None: ...
    debug_sql_stream: StringIO = ...
    handler: logging.FileHandler = ...
    def startTest(self, test: _TestCase) -> None: ...
    def stopTest(self, test: _TestCase) -> None: ...
    def addError(self, test: Any, err: Any) -> None: ...
    def addFailure(self, test: Any, err: Any) -> None: ...

class RemoteTestResult:
    events: List[Any] = ...
    failfast: bool = ...
    shouldStop: bool = ...
    testsRun: int = ...
    def __init__(self) -> None: ...
    @property
    def test_index(self) -> Any: ...
    def check_picklable(self, test: Any, err: Any) -> None: ...
    def _confirm_picklable(self, obj: Any) -> None: ...
    def check_subtest_picklable(self, test: Any, subtest: Any) -> None: ...
    def stop_if_failfast(self) -> None: ...
    def stop(self) -> None: ...
    def startTestRun(self) -> None: ...
    def stopTestRun(self) -> None: ...
    def startTest(self, test: Any) -> None: ...
    def stopTest(self, test: Any) -> None: ...
    def addError(self, test: Any, err: Any) -> None: ...
    def addFailure(self, test: Any, err: Any) -> None: ...
    def addSubTest(self, test: Any, subtest: Any, err: Any) -> None: ...
    def addSuccess(self, test: Any) -> None: ...
    def addSkip(self, test: Any, reason: Any) -> None: ...
    def addExpectedFailure(self, test: Any, err: Any) -> None: ...
    def addUnexpectedSuccess(self, test: Any) -> None: ...

class RemoteTestRunner:
    resultclass: Any = ...
    failfast: Any = ...
    def __init__(
        self, failfast: bool = ..., resultclass: Optional[Any] = ...
    ) -> None: ...
    def run(self, test: Any) -> Any: ...

def default_test_processes() -> int: ...

class ParallelTestSuite(TestSuite):
    init_worker: Any = ...
    run_subsuite: Any = ...
    runner_class: Any = ...
    subsuites: Any = ...
    processes: Any = ...
    failfast: Any = ...
    def __init__(self, suite: Any, processes: Any, failfast: bool = ...) -> None: ...
    def run(self, result: Any) -> Any: ...  # type: ignore[override]

class DiscoverRunner:
    test_suite: Any = ...
    parallel_test_suite: Any = ...
    test_runner: Any = ...
    test_loader: Any = ...
    reorder_by: Any = ...
    pattern: Optional[str] = ...
    top_level: None = ...
    verbosity: int = ...
    interactive: bool = ...
    failfast: bool = ...
    keepdb: bool = ...
    reverse: bool = ...
    debug_mode: bool = ...
    debug_sql: bool = ...
    parallel: int = ...
    tags: Set[str] = ...
    exclude_tags: Set[str] = ...
    def __init__(
        self,
        pattern: Optional[str] = ...,
        top_level: None = ...,
        verbosity: int = ...,
        interactive: bool = ...,
        failfast: bool = ...,
        keepdb: bool = ...,
        reverse: bool = ...,
        debug_mode: bool = ...,
        debug_sql: bool = ...,
        parallel: int = ...,
        tags: Optional[List[str]] = ...,
        exclude_tags: Optional[List[str]] = ...,
        **kwargs: Any
    ) -> None: ...
    @classmethod
    def add_arguments(cls, parser: ArgumentParser) -> None: ...
    def setup_test_environment(self, **kwargs: Any) -> None: ...
    def build_suite(
        self,
        test_labels: Sequence[str] = ...,
        extra_tests: Optional[List[Any]] = ...,
        **kwargs: Any
    ) -> TestSuite: ...
    def setup_databases(
        self, **kwargs: Any
    ) -> List[Tuple[BaseDatabaseWrapper, str, bool]]: ...
    def get_resultclass(self) -> Optional[Type[DebugSQLTextTestResult]]: ...
    def get_test_runner_kwargs(self) -> Dict[str, Optional[int]]: ...
    def run_checks(self) -> None: ...
    def run_suite(self, suite: TestSuite, **kwargs: Any) -> TextTestResult: ...
    def teardown_databases(
        self, old_config: List[Tuple[BaseDatabaseWrapper, str, bool]], **kwargs: Any
    ) -> None: ...
    def teardown_test_environment(self, **kwargs: Any) -> None: ...
    def suite_result(
        self, suite: TestSuite, result: TextTestResult, **kwargs: Any
    ) -> int: ...
    def run_tests(
        self, test_labels: List[str], extra_tests: List[Any] = ..., **kwargs: Any
    ) -> int: ...

def is_discoverable(label: str) -> bool: ...
def reorder_suite(
    suite: TestSuite,
    classes: Tuple[Type[TestCase], Type[SimpleTestCase]],
    reverse: bool = ...,
) -> TestSuite: ...
def partition_suite_by_type(
    suite: TestSuite,
    classes: Tuple[Type[TestCase], Type[SimpleTestCase]],
    bins: List[OrderedSet[Any]],
    reverse: bool = ...,
) -> None: ...
def partition_suite_by_case(suite: Any) -> Any: ...
def filter_tests_by_tags(
    suite: TestSuite, tags: Set[str], exclude_tags: Set[str]
) -> TestSuite: ...



import abc
from typing import (
    Any,
    Iterator,
)

ResType = dict[int, Any]

def frame_apply(
    obj: DataFrame,
    func,
    axis=...,
    raw: bool = ...,
    result_type=...,
    ignore_failures: bool = ...,
    args=...,
    kwds=...,
): ...

class FrameApply(metaclass=abc.ABCMeta):
    axis: int
    @property
    @abc.abstractmethod
    def result_index(self) -> Index: ...
    @property
    @abc.abstractmethod
    def result_columns(self) -> Index: ...
    @property
    @abc.abstractmethod
    def series_generator(self) -> Iterator[Series]: ...
    @abc.abstractmethod
    def wrap_results_for_axis(
        self, results: ResType, res_index: Index
    ) -> Series | DataFrame: ...
    obj = ...
    raw = ...
    ignore_failures = ...
    args = ...
    kwds = ...
    result_type = ...
    f = ...
    def __init__(
        self,
        obj: DataFrame,
        func,
        raw: bool,
        result_type,
        ignore_failures: bool,
        args,
        kwds,
    ): ...
    @property
    def res_columns(self) -> Index: ...
    @property
    def columns(self) -> Index: ...
    @property
    def index(self) -> Index: ...
    def values(self): ...
    def dtypes(self) -> Series: ...
    @property
    def agg_axis(self) -> Index: ...
    def get_result(self): ...
    def apply_empty_result(self): ...
    def apply_raw(self): ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    def apply_standard(self): ...
    def apply_series_generator(self) -> tuple[ResType, Index]: ...
    def wrap_results(
        self, results: ResType, res_index: Index
    ) -> Series | DataFrame: ...

class FrameRowApply(FrameApply):
    axis: int = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    @property
    def series_generator(self): ...
    @property
    def result_index(self) -> Index: ...
    @property
    def result_columns(self) -> Index: ...
    def wrap_results_for_axis(
        self, results: ResType, res_index: Index
    ) -> DataFrame: ...

class FrameColumnApply(FrameApply):
    axis: int = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame: ...
    @property
    def series_generator(self): ...
    @property
    def result_index(self) -> Index: ...
    @property
    def result_columns(self) -> Index: ...
    def wrap_results_for_axis(
        self, results: ResType, res_index: Index
    ) -> Series | DataFrame: ...
    def infer_to_same_shape(self, results: ResType, res_index: Index) -> DataFrame: ...

from pandas.core.frame import DataFrame
from pandas.core.indexes.base import Index
from pandas.core.series import Series



from typing import Any, Dict, Optional, Tuple, Union

from django.db.backends.base.base import BaseDatabaseWrapper

TEST_DATABASE_PREFIX: str

class BaseDatabaseCreation:
    connection: Any = ...
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def create_test_db(
        self,
        verbosity: int = ...,
        autoclobber: bool = ...,
        serialize: bool = ...,
        keepdb: bool = ...,
    ) -> str: ...
    def set_as_test_mirror(
        self,
        primary_settings_dict: Dict[str, Optional[Union[Dict[str, None], int, str]]],
    ) -> None: ...
    def serialize_db_to_string(self) -> str: ...
    def deserialize_db_from_string(self, data: str) -> None: ...
    def clone_test_db(
        self,
        suffix: Any,
        verbosity: int = ...,
        autoclobber: bool = ...,
        keepdb: bool = ...,
    ) -> None: ...
    def get_test_db_clone_settings(self, suffix: Any) -> Any: ...
    def destroy_test_db(
        self,
        old_database_name: str = ...,
        verbosity: int = ...,
        keepdb: bool = ...,
        suffix: None = ...,
    ) -> None: ...
    def sql_table_creation_suffix(self) -> Any: ...
    def test_db_signature(self) -> Tuple[str, str, str, str]: ...
